package handler

import (
	"errors"
	"github.com/violendy/SalesPortal/internal/models"
	"log"
	"strconv"
)

func (rp *Repository) DoPortalGetAllCustomer(User models.UserData, params interface{}) (error, int, interface{}) {
	CustomerList, err := rp.PortalDB.CustomerListGet(User.KodePT)
	if err != nil {
		return err, 0, nil
	}
	return nil, len(*CustomerList), CustomerList
}
func (rp *Repository) DoPortalGetCustomerRecordByKodeCust(User models.UserData, params interface{}) (error, int, interface{}) {
	sKodeCust := params.(string)
	CustomerList, recCount, err := rp.PortalDB.CustomerGetByKodeCust(User.KodePT, sKodeCust)
	if err != nil {
		return err, 0, nil
	}
	return nil, recCount, CustomerList
}
func (rp *Repository) DoPortalCreateCustomerRecord(User models.UserData, UserStruct interface{}, params interface{}) (error, int, interface{}) {
	var a = UserStruct.(*models.CustomerData)
	NewUserRecord, err := rp.PortalDB.CustomerAdd(User.KodePT, *a)
	if err != nil {
		return err, 0, nil
	}
	return nil, 1, NewUserRecord
}
func (rp *Repository) DoPortalUpdateCustomerRecord(User models.UserData, UserStruct interface{}, params interface{}) (error, int, interface{}) {
	var a = UserStruct.(*models.CustomerData)
	NewUserRecord, recCount, err := rp.PortalDB.CustomerUpdate(User.KodePT, *a)
	if err != nil {
		return err, 0, nil
	}
	return nil, recCount, NewUserRecord
}
func (rp *Repository) DoPortalDeleteCustomerRecord(User models.UserData, params interface{}) (error, int, interface{}) {
	sUIDDelete := params.(string)
	err := rp.PortalDB.CustomerDelete(User.KodePT, sUIDDelete)
	if err != nil {
		return err, 0, nil
	}
	return nil, 1, nil
}
func (rp *Repository) DoPortalApprovePayment(User models.UserData, params interface{}) (error, int, interface{}) {
	sUIDApprove := params.(string)
	err, approved := rp.PortalDB.PaymentApprove(sUIDApprove, models.Approved)
	if err != nil {
		return err, 0, nil
	}
	if *approved == models.Approved {

		Payment, _, err := rp.PortalDB.PaymentGetByUID(sUIDApprove)
		if err != nil {
			return err, 0, nil
		}
		err = rp.NKDB.PaymentAdd(User.KodePT, *Payment)
		if err != nil {
			return err, 0, nil
		}
		return nil, 1, approved
	} else if *approved != models.NoChange {
		return errors.New("no data to update"), 0, nil
	}
	return nil, 0, nil
}

func (rp *Repository) DoPortalApproveCustomer(User models.UserData, params interface{}) (error, int, interface{}) {
	sUIDApprove := params.(string)
	err, approved := rp.PortalDB.CustomerApprove(sUIDApprove, models.Approved)
	if err != nil {
		return err, 0, nil
	}
	if *approved == models.Approved {
		Customer, _, err := rp.PortalDB.CustomerGetByUID(sUIDApprove)
		if err != nil {
			return err, 0, nil
		}
		err = rp.NKDB.CustomerAdd(User.KodePT, *Customer)
		if err != nil {
			return err, 0, nil
		}
		return nil, 1, approved
	} else if *approved != models.NoChange {
		return errors.New("no data to update"), 0, nil
	}
	return nil, 0, nil
}
func (rp *Repository) DoSalesChangePassword(Sales models.SalesData, UserStruct interface{}, params interface{}) (error, int, interface{}) {
	var NewCredential = UserStruct.(*models.ChangeCredential)
	SalesRecord, err := rp.PortalDB.ChangePassword(Sales.KodePT, NewCredential)
	if err != nil {
		return err, 0, nil
	}
	return nil, 1, SalesRecord
}

func (rp *Repository) DoSalesGetRKSList(Sales models.SalesData, params interface{}) (error, int, interface{}) {
	ClientSyncID, _ := rp.NKDB.GetSyncSales(Sales.KodePT, "RKS", Sales.KodeSales)
	NeedToSync, _ := rp.PortalDB.NeedSync(Sales.KodePT, Sales.KodeSales, "RKS", ClientSyncID)
	if !NeedToSync {
		// Data RKS masih up to date
		RKS, err := rp.PortalDB.RKSListGetBySales(Sales.KodePT, Sales.KodeSales)
		if err != nil {
			return err, 0, nil
		} else {
			return nil, len(*RKS), RKS
		}
	} else {
		// Ambil RKS dari database client sesuai cabang (KodePT)
		RKSSource, err := rp.NKDB.RKSListGet(Sales.KodePT)
		if err != nil {
			return errors.New("Can't get RKS Data from source database"), 0, nil
		}
		NewRKSRecCount, err := rp.PortalDB.RKSSetData(Sales.KodePT, RKSSource)
		if err != nil {
			return errors.New("Gagal mengisi RKS dari client ke portal"), 0, nil
		} else {
			err := rp.PortalDB.UpdateSyncID(Sales.KodePT, Sales.KodeSales, "RKS", ClientSyncID)
			if err != nil {
				log.Println("Error update sync ID RKS : ", err)
			}
			if NewRKSRecCount > 0 {
				NewRKS, err := rp.PortalDB.RKSListGetBySales(Sales.KodePT, Sales.KodeSales)
				if err != nil {
					return errors.New("Can't get RKS Data from portal database"), 0, nil
				} else {
					return nil, len(*NewRKS), NewRKS
				}
			}
		}
	}
	return nil, 0, nil
}
func (rp *Repository) DoSalesGetCustomerList(Sales models.SalesData, params interface{}) (error, int, interface{}) {
	ClientSyncID, _ := rp.NKDB.GetSyncSales(Sales.KodePT, "Customer", Sales.KodeSales)
	NeedToSync, _ := rp.PortalDB.NeedSync(Sales.KodePT, Sales.KodeSales, "Customer", ClientSyncID)
	if !NeedToSync {
		// Data Customer masih up to date
		Customer, err := rp.PortalDB.CustomerListGetBySales(Sales.KodePT, Sales.KodeSales)
		if err != nil {
			return errors.New("Gagal ambil List Customer dari database portal"), 0, nil
		} else {
			return nil, len(*Customer), Customer
		}
	} else {
		// Ambil Data Customer dari database client sesuai cabang (KodePT)
		CustomerSource, err := rp.NKDB.CustomerListGetBySales(Sales.KodePT, Sales.KodeSales)
		if err != nil {
			return errors.New("Can't get Customer Data from source database"), 0, nil
		}
		NewCustRecCount, err := rp.PortalDB.CustomerSet(Sales.KodePT, CustomerSource)
		if err != nil {
			return errors.New("Gagal mengisi Customer dari client ke portal"), 0, nil
		} else {
			err := rp.PortalDB.UpdateSyncID(Sales.KodePT, Sales.KodeSales, "Customer", ClientSyncID)
			if err != nil {
				log.Println(err)
			}
			if NewCustRecCount > 0 {
				NewCustomerData, err := rp.PortalDB.CustomerListGetBySales(Sales.KodePT, Sales.KodeSales)
				if err != nil {
					return errors.New("Gagal ambil Customer dari database portal"), 0, nil
				} else {
					return nil, len(*NewCustomerData), NewCustomerData
				}
			}
		}
	}
	return nil, 0, nil
}
func (rp *Repository) DoSalesGetCustomerRecordByKodeCust(Sales models.SalesData, params interface{}) (error, int, interface{}) {
	sKodeCust := params.(string)
	ClientSyncID, _ := rp.NKDB.GetSyncSales(Sales.KodePT, "Customer", Sales.KodeSales)
	NeedToSync, _ := rp.PortalDB.NeedSync(Sales.KodePT, Sales.KodeSales, "Customer", ClientSyncID)
	if !NeedToSync {
		// Data Customer masih up to date

		Customer, nCount, err := rp.PortalDB.CustomerGetBySalesKodeCust(Sales.KodePT, Sales.KodeSales, sKodeCust)
		if err != nil {
			return errors.New("Gagal ambil Data Customer dari database portal"), 0, nil
		} else {
			return nil, nCount, Customer
		}
	} else {
		// Ambil Data Customer dari database client sesuai cabang (KodePT)
		CustomerSource, err := rp.NKDB.CustomerListGetBySales(Sales.KodePT, Sales.KodeSales)
		if err != nil {
			return errors.New("Can't get Customer Data from source database"), 0, nil
		}
		NewCustRecCount, err := rp.PortalDB.CustomerSet(Sales.KodePT, CustomerSource)
		if err != nil {
			return errors.New("Gagal mengisi Customer dari client ke portal"), 0, nil

		} else {
			err := rp.PortalDB.UpdateSyncID(Sales.KodePT, Sales.KodeSales, "Customer", ClientSyncID)
			if err != nil {
				log.Print(err)
			}
			if NewCustRecCount > 0 {
				NewCustomerData, nCount, err := rp.PortalDB.CustomerGetBySalesKodeCust(Sales.KodePT, Sales.KodeSales, sKodeCust)
				if err != nil {
					return errors.New("Gagal ambil Customer dari database portal"), 0, nil
				} else {
					return nil, nCount, NewCustomerData
				}
			}
		}
	}
	return nil, 0, nil
}
func (rp *Repository) DoSalesCreateCustomerRecord(Sales models.SalesData, UserStruct interface{}, params interface{}) (error, int, interface{}) {
	var NewCustomerRecord = UserStruct.(*models.CustomerData)
	customer, err := rp.PortalDB.CustomerAdd(Sales.KodePT, *NewCustomerRecord)
	if err != nil {
		return err, 0, nil
	}
	return nil, 1, customer
}
func (rp *Repository) DoGetCurrentCustomerLocation(Sales models.SalesData, params interface{}) (error, int, interface{}) {

	if SalesLocationStr, ok := params.(map[string]string); ok {
		salesLat, err := strconv.ParseFloat(SalesLocationStr["lat"], 64)
		if err != nil {
			return errors.New("bad request"), 0, nil
		}
		salesLng, err := strconv.ParseFloat(SalesLocationStr["lng"], 64)
		if err != nil {
			return errors.New("bad request"), 0, nil
		}
		SalesLocation := models.Location{
			Lat: salesLat,
			Lng: salesLng,
		}
		err, recCount, PossibleCustomer := rp.PortalDB.SalesGetCustomerInArea(Sales.KodePT, Sales.KodeSales, SalesLocation)
		if err != nil {
			return errors.New("bad request"), 0, nil
		}
		return nil, recCount, PossibleCustomer
	} else {
		return errors.New("bad request"), 0, nil
	}

}

func (rp *Repository) DoSalesUpdateCustomerRecord(Sales models.SalesData, UserStruct interface{}, params interface{}) (error, int, interface{}) {
	var NewCustomerRecord = UserStruct.(*models.CustomerData)
	log.Println("DoSalesUpdateCustomerRecord")
	CustomerRecord, recCount, _ := rp.PortalDB.CustomerGetBySalesKodeCust(Sales.KodePT, Sales.KodeSales, NewCustomerRecord.KodeCust)
	if recCount == 0 {
		log.Println("No customer exists " + NewCustomerRecord.KodeCust)
		return errors.New("Error updating customer, no record exists"), recCount, nil
	} else {
		CustomerRecord.GPSLocation.Lat = NewCustomerRecord.GPSLocation.Lat
		CustomerRecord.GPSLocation.Lng = NewCustomerRecord.GPSLocation.Lng
		CustomerRecord.NamaCust = NewCustomerRecord.NamaCust
		CustomerRecord.AlamatCust = NewCustomerRecord.AlamatCust
		CustomerRecord.Kecamatan = NewCustomerRecord.Kecamatan
		CustomerRecord.Kota = NewCustomerRecord.Kota
		CustomerRecord.NoKTP = NewCustomerRecord.NoKTP
		CustomerRecord.FileFotoKTP = NewCustomerRecord.FileFotoKTP
		CustomerRecord.FileFotoToko1 = NewCustomerRecord.FileFotoToko1
		CustomerRecord.FileFotoToko2 = NewCustomerRecord.FileFotoToko2
		CustomerRecord.FileFotoToko3 = NewCustomerRecord.FileFotoToko3

		customer, updatedCustCount, err := rp.PortalDB.CustomerUpdate(Sales.KodePT, *CustomerRecord)
		if err != nil {
			return errors.New("Error updating customer to database"), 0, nil
		} else {
			log.Println(customer)
			return nil, updatedCustCount, customer
		}
	}
}

func (rp *Repository) DoSalesCheckIn(Sales models.SalesData, UserStruct interface{}, params interface{}) (error, int, interface{}) {
	var a = UserStruct.(*models.SalesCheckIn)
	CheckInRecord, err := rp.PortalDB.SalesCheckIn(Sales.KodePT, *a)
	return err, 1, CheckInRecord
}

func (rp *Repository) DoSalesCheckOut(Sales models.SalesData, UserStruct interface{}, params interface{}) (error, int, interface{}) {
	var a = UserStruct.(*models.SalesCheckOut)
	CheckOutRecord, err := rp.PortalDB.SalesCheckOut(Sales.KodePT, *a)
	return err, 1, CheckOutRecord
}
func (rp *Repository) DoSalesGetSalesOrderList(Sales models.SalesData, params interface{}) (error, int, interface{}) {
	SalesOrder, err := rp.PortalDB.SalesOrderListGetBySales(Sales.KodePT, Sales.KodeSales)
	if err != nil {
		return err, 0, nil
	}
	return nil, len(*SalesOrder), SalesOrder
}
func (rp *Repository) DoGetVisit(Sales models.SalesData, params interface{}) (error, int, interface{}) {
	SalesVisit, recCount, err := rp.PortalDB.SalesGetVisit(Sales.KodePT, Sales.KodeSales)
	if err != nil {
		return err, 0, nil
	}
	return nil, recCount, SalesVisit
}
func (rp *Repository) DoGetProduct(Sales models.SalesData, params interface{}) (error, int, interface{}) {
	ProductList, recCount, err := rp.PortalDB.SalesGetProduct(Sales.KodePT)
	if err != nil {
		return err, 0, nil
	}
	return nil, recCount, ProductList
}
func (rp *Repository) DoSalesGetSalesOrderRecord(Sales models.SalesData, params interface{}) (error, int, interface{}) {
	sNoOrder := params.(string)
	SalesOrder, recCount, err := rp.PortalDB.SalesOrderGetByNoOrder(Sales.KodePT, sNoOrder, Sales.KodeSales)
	if err != nil {
		return err, 0, nil
	}
	return nil, recCount, SalesOrder
}
func (rp *Repository) DoSalesCreateSalesOrderRecord(Sales models.SalesData, UserStruct interface{}, params interface{}) (error, int, interface{}) {
	var a = UserStruct.(*models.OrderData)
	SalesRecord, recCount, err := rp.PortalDB.SalesOrderAdd(Sales.KodePT, Sales.KodeSales, *a)
	return err, recCount, SalesRecord
}
func (rp *Repository) DoDeleteOrder(Sales models.SalesData, params interface{}) (error, int, interface{}) {
	sUID := params.(string)
	err := rp.PortalDB.SalesOrderDelete(Sales.KodePT, sUID)
	return err, 0, nil
}
func (rp *Repository) DoDeletePayment(Sales models.SalesData, params interface{}) (error, int, interface{}) {
	sUID := params.(string)
	err := rp.PortalDB.SalesPaymentDelete(Sales.KodePT, sUID)
	return err, 0, nil
}
func (rp *Repository) DoSalesUpdateSalesOrderRecord(Sales models.SalesData, UserStruct interface{}, params interface{}) (error, int, interface{}) {
	var a = UserStruct.(*models.OrderData)
	SalesRecord, recCount, err := rp.PortalDB.SalesOrderUpdate(Sales.KodePT, Sales.KodeSales, *a)
	return err, recCount, SalesRecord
}
func (rp *Repository) DoSalesGetPaymentList(Sales models.SalesData, params interface{}) (error, int, interface{}) {
	PaymentList, err := rp.PortalDB.PaymentListGetBySales(Sales.KodePT, Sales.KodeSales)
	if err != nil {
		return err, 0, nil
	}
	return nil, len(*PaymentList), PaymentList
}
func (rp *Repository) DoSalesGetPaymentRecord(Sales models.SalesData, params interface{}) (error, int, interface{}) {
	sNoTransaksi := params.(string)
	PaymentRecord, recCount, err := rp.PortalDB.PaymentGetBySalesNoTransaksi(Sales.KodePT, sNoTransaksi, Sales.KodeSales)
	if err != nil {
		return err, 0, nil
	}
	return nil, recCount, PaymentRecord
}
func (rp *Repository) DoSalesCreatePaymentRecord(Sales models.SalesData, UserStruct interface{}, params interface{}) (error, int, interface{}) {
	var a = UserStruct.(*models.PaymentData)
	log.Println(Sales.KodePT, Sales.KodeSales)
	PaymentRecord, recCount, err := rp.PortalDB.PaymentAdd(Sales.KodePT, Sales.KodeSales, *a)
	return err, recCount, PaymentRecord
}
func (rp *Repository) DoSalesUpdatePaymentRecord(Sales models.SalesData, UserStruct interface{}, params interface{}) (error, int, interface{}) {
	var a = UserStruct.(*models.PaymentData)
	PaymentRecord, recCount, err := rp.PortalDB.PaymentUpdate(Sales.KodePT, Sales.KodeSales, *a)
	return err, recCount, PaymentRecord
}
