package handler

import (
	"crypto/sha1"
	"encoding/base64"
	"encoding/hex"
	"errors"
	"fmt"
	"github.com/go-chi/chi/v5"
	"github.com/mholt/binding"
	"github.com/violendy/SalesPortal/internal/models"
	"io"
	"io/ioutil"
	"log"
	"net/http"
	"os"
	"path/filepath"
	"strconv"
	"time"
)

func (rp *Repository) Home(w http.ResponseWriter, r *http.Request) {
	//rp.EnableCors(&w)
	if r.Method == "GET" {
		_, err := fmt.Fprintf(w, "Server Aktif!")
		if err != nil {
			log.Print(err)
			return
		}
	}
}

func (rp *Repository) ToggleProduction(w http.ResponseWriter, r *http.Request) {
	rp.App.InProduction = !rp.App.InProduction
	_, err := fmt.Fprintf(w, "isProduction : "+strconv.FormatBool(rp.App.InProduction))
	if err != nil {
		log.Print(err)
		return
	}
}

func (rp *Repository) SetContextTimeout(w http.ResponseWriter, r *http.Request) {
	var newValue, _ = strconv.Atoi(chi.URLParam(r, "val"))
	rp.App.ContextValue = time.Duration(newValue)
	_, err := fmt.Fprintf(w, "TimeOut : "+rp.App.ContextValue.String())
	if err != nil {
		log.Print(err)
		return
	}
}

func (rp *Repository) FileUpload(w http.ResponseWriter, r *http.Request) {
	//rp.EnableCors(&w)
	sAuth := r.Header.Get("Authorization")
	if sAuth != "" {
		_, bAuthenticated := rp.AuthenticateSales(sAuth)
		if bAuthenticated {

			r.ParseMultipartForm(32 << 20)

			file, handler, err := r.FormFile("file")

			if err != nil {
				log.Println(err)
				w.WriteHeader(http.StatusInternalServerError)
				return
			}
			defer file.Close()

			hash := sha1.New()
			if _, err := io.Copy(hash, file); err != nil {
				rp.ServerErrorResponse(w, "Gagal upload file!", "Gagal Upload File", "", err.Error(), http.StatusInternalServerError)
				return
			}
			checksum := hex.EncodeToString(hash.Sum(nil))
			storedFileName, err := rp.PortalDB.NewFile(handler.Filename, checksum)
			if err != nil {
				w.WriteHeader(http.StatusInternalServerError)
				rp.ServerErrorResponse(w, "Gagal upload file!", "Gagal Upload File", "", err.Error(), http.StatusInternalServerError)
				return
			}

			wd, err := os.Getwd()
			f, err := os.OpenFile(wd+"\\uploaded\\"+storedFileName, os.O_WRONLY|os.O_CREATE, 0666)

			if err != nil {

				rp.ServerErrorResponse(w, "Gagal upload file!", "Gagal Upload File", "", err.Error(), http.StatusInternalServerError)
				return
			}
			defer f.Close()

			_, err = file.Seek(0, io.SeekStart)
			_, err = io.Copy(f, file)
			if err != nil {
				w.WriteHeader(http.StatusInternalServerError)
				return
			}
			if storedFileName != "" {
				w.WriteHeader(http.StatusOK)
				w.Write([]byte(fmt.Sprintf("{\"file_id\": \"%s\"}", checksum)))
			}

		} else {
			rp.ServerErrorResponse(w, "Gagal upload file!", "Gagal Upload File", "", errors.New("invalid authentication header"), http.StatusUnauthorized)
			return
		}
	}
}
func (rp *Repository) PortalApproveResetPassword(w http.ResponseWriter, r *http.Request) {
	//rp.EnableCors(&w)

}

func (rp *Repository) UserLogin(w http.ResponseWriter, r *http.Request) {
	//rp.EnableCors(&w)
	LoginCreds := new(models.LoginCredential)
	if errs := binding.Bind(r, LoginCreds); errs != nil {
		rp.ServerErrorResponse(w, "Bad Request", "Gagal binding request ke Type SalesLogin Credential", "", LoginCreds, http.StatusBadRequest)
		return
	} else {
		var LoginResp models.UserLoginResponse
		LoginResp, err := rp.PortalDB.UserLogin(LoginCreds)
		if err != nil {
			rp.ServerErrorResponse(w, "Bad Username / Password", "Login gagal", "", err.Error(), http.StatusUnauthorized)
			return
		} else {
			rp.ServerResponse(w, "Login berhasil!", 1, LoginResp)
			return
		}
	}
}
func (rp *Repository) UserLogout(w http.ResponseWriter, r *http.Request) {
	//rp.EnableCors(&w)
	sAuth := r.Header.Get("Authorization")
	if sAuth != "" {
		Sales, bAuthenticated := rp.AuthenticateSales(sAuth)
		if bAuthenticated {
			Succeed, err := rp.PortalDB.SalesLogout(Sales.KodeSales)
			if err != nil || !Succeed {
				Resp := models.LogoutStatus{
					Login: models.LoggedIn.String(),
					Msg:   "Function SalesLogout returns LoggedIn",
				}
				rp.ServerErrorResponse(w, "Call SalesLogout Error", "Function SalesLogout Returns LoggedIn", "", Resp, http.StatusInternalServerError)
				return
			} else {
				Resp := models.LogoutStatus{
					Login: models.LoggedOut.String(),
					Msg:   "LoggedOut",
				}
				rp.ServerResponse(w, "SalesLogout Succeed", 1, Resp)
				return
			}
		} else {
			Resp := models.LogoutStatus{
				Login: models.LoggedOut.String(),
				Msg:   "ExpiredAuth",
			}
			rp.ServerResponse(w, "ExpiredOrInvalidAuth", 1, Resp)
			return
		}
	} else {
		rp.ServerErrorResponse(w, "Error logout!", "Bad Request", "", errors.New("no authentication header").Error(), http.StatusBadRequest)
		return
	}
}

func (rp *Repository) PortalGetAllCustomerList(w http.ResponseWriter, r *http.Request) {
	//rp.EnableCors(&w)
	sr := GenerateServerResponse("CustomerList", GET)
	rp.ProcessPortalRequest(w, r, rp.DoPortalGetAllCustomer, nil, sr)
}
func (rp *Repository) PortalGetCustomerRecordByKodeCust(w http.ResponseWriter, r *http.Request) {
	//rp.EnableCors(&w)
	sr := GenerateServerResponse("CustomerRecord", GET)
	sKodeCust := chi.URLParam(r, "KodeCust")
	rp.ProcessPortalRequest(w, r, rp.DoPortalGetCustomerRecordByKodeCust, sKodeCust, sr)
}
func (rp *Repository) PortalCreateCustomerRecord(w http.ResponseWriter, r *http.Request) {
	//rp.EnableCors(&w)
	sr := GenerateServerResponse("CustomerRecord", POST)
	rp.ProcessPortalRequestWithBinding(w, r, rp.DoPortalCreateCustomerRecord, &models.CustomerData{}, nil, sr)
}
func (rp *Repository) PortalUpdateCustomerRecord(w http.ResponseWriter, r *http.Request) {
	//rp.EnableCors(&w)
	sr := GenerateServerResponse("CustomerRecord", PUT)
	rp.ProcessPortalRequestWithBinding(w, r, rp.DoPortalUpdateCustomerRecord, &models.CustomerData{}, nil, sr)
}
func (rp *Repository) PortalApproveCustomer(w http.ResponseWriter, r *http.Request) {
	//rp.EnableCors(&w)
	sr := GenerateServerResponse("CustomerApproval", PUT)
	sUID := chi.URLParam(r, "UID")
	rp.ProcessPortalRequest(w, r, rp.DoPortalApproveCustomer, sUID, sr)
}
func (rp *Repository) SalesGetCurrentLocationCustomer(w http.ResponseWriter, r *http.Request) {
	//rp.EnableCors(&w)
	sr := GenerateServerResponse("CustomerByLocation", GET)
	salesLat := chi.URLParam(r, "salesLat")
	salesLng := chi.URLParam(r, "salesLng")
	SalesLocation := map[string]string{
		"lat": salesLat,
		"lng": salesLng,
	}
	rp.ProcessSalesRequest(w, r, rp.DoGetCurrentCustomerLocation, SalesLocation, sr)
}
func (rp *Repository) SendWhatsApp(w http.ResponseWriter, r *http.Request) {
	sNoHP := chi.URLParam(r, "nohp")
	sMessage := chi.URLParam(r, "message")
	rp.WA.SendMessage(sNoHP, sMessage)
}
func (rp *Repository) PortalApprovePayment(w http.ResponseWriter, r *http.Request) {
	//rp.EnableCors(&w)
	sr := GenerateServerResponse("PaymentApproval", PUT)
	sUID := chi.URLParam(r, "UID")
	rp.ProcessPortalRequest(w, r, rp.DoPortalApprovePayment, sUID, sr)
}
func (rp *Repository) DeleteCustomerRecord(w http.ResponseWriter, r *http.Request) {
	//rp.EnableCors(&w)
	sr := GenerateServerResponse("CustomerRecord", DELETE)
	sUIDDelete := chi.URLParam(r, "ResourceUID")
	rp.ProcessPortalRequest(w, r, rp.DoPortalDeleteCustomerRecord, sUIDDelete, sr)
}

func (rp *Repository) SalesLogin(w http.ResponseWriter, r *http.Request) {
	//rp.EnableCors(&w)
	//rp.EnableCors(&w)
	LoginCreds := new(models.LoginCredential)
	if errs := binding.Bind(r, LoginCreds); errs != nil {
		rp.ServerErrorResponse(w, "Bad Request", "Gagal binding request ke Type SalesLogin Credential", "", LoginCreds, http.StatusBadRequest)
		return
	} else {
		var LoginResp models.SalesLoginResponse
		LoginResp, err := rp.PortalDB.SalesLogin(LoginCreds)
		if err != nil {
			rp.ServerErrorResponse(w, "Bad Username / Password", "Gagal login", "", err.Error(), http.StatusUnauthorized)
			return
		} else {
			rp.ServerResponse(w, "SalesLogin berhasil!", 1, LoginResp)
			return
		}
	}
}
func (rp *Repository) SalesLogout(w http.ResponseWriter, r *http.Request) {
	//rp.EnableCors(&w)
	sAuth := r.Header.Get("Authorization")
	if sAuth != "" {
		Sales, bAuthenticated := rp.AuthenticateSales(sAuth)
		if bAuthenticated {
			Succeed, err := rp.PortalDB.SalesLogout(Sales.KodeSales)
			if err != nil || !Succeed {
				Resp := models.LogoutStatus{
					Login: models.LoggedIn.String(),
					Msg:   "Function SalesLogout returns LoggedIn",
				}
				rp.ServerErrorResponse(w, "Call SalesLogout Error", "Function SalesLogout Returns LoggedIn", "", Resp, http.StatusInternalServerError)
				return
			} else {
				Resp := models.LogoutStatus{
					Login: models.LoggedOut.String(),
					Msg:   "LoggedOut",
				}
				rp.ServerResponse(w, "SalesLogout Succeed", 1, Resp)
				return
			}
		} else {
			Resp := models.LogoutStatus{
				Login: models.LoggedOut.String(),
				Msg:   "ExpiredAuth",
			}
			rp.ServerResponse(w, "ExpiredOrInvalidAuth", 1, Resp)
			return
		}
	} else {
		rp.ServerErrorResponse(w, "Error logout!", "Bad Request", "", errors.New("no authentication header").Error(), http.StatusBadRequest)
		return
	}
}
func (rp *Repository) SalesChangePassword(w http.ResponseWriter, r *http.Request) {
	//rp.EnableCors(&w)
	sr := GenerateServerResponse("SalesLogin", POST)
	rp.ProcessSalesRequestWithBinding(w, r, rp.DoSalesChangePassword, &models.ChangeCredential{}, nil, sr)
}

func (rp *Repository) SalesRequestResetPassword(w http.ResponseWriter, r *http.Request) {
	noHP := rp.WA.FormatPhoneToJID(chi.URLParam(r, "NoHP"))
	NewPassword, KodeSales, err := rp.PortalDB.RequestResetPassword(noHP)
	if err != nil {
		rp.ServerErrorResponse(w, "Proses reset password gagal. Pastikan nomor handphone Anda terdaftar.", "Gagal reset", "", err.Error(), http.StatusBadRequest)
	} else {
		err = rp.WA.SendMessage(noHP, "Password untuk user: "+KodeSales+" berhasil diubah. Password baru: "+NewPassword)
		if err != nil {
			rp.ServerErrorResponse(w, "Gagal mengirim password baru via WhatsApp", "Gagal reset", "", err.Error(), http.StatusBadRequest)
			return
		} else {
			rp.ServerResponse(w, "Proses reset password sukses", 0, "")
			return
		}

	}

}
func (rp *Repository) SalesGetRKSList(w http.ResponseWriter, r *http.Request) {
	//rp.EnableCors(&w)
	sr := GenerateServerResponse("RKSList", GET)
	rp.ProcessSalesRequest(w, r, rp.DoSalesGetRKSList, nil, sr)
}

func (rp *Repository) SalesGetPicture(w http.ResponseWriter, r *http.Request) {
	PictureID := chi.URLParam(r, "filename")
	path := filepath.Join(".", "UploadedPhotos", PictureID)
	file, err := os.Open(path)
	if err != nil {
		http.Error(w, err.Error(), http.StatusInternalServerError)
		return
	}
	defer file.Close()

	// Read the file into a byte slice
	data, err := ioutil.ReadAll(file)
	if err != nil {
		http.Error(w, err.Error(), http.StatusInternalServerError)
		return
	}

	// Encode the byte slice to base64
	encoded := base64.StdEncoding.EncodeToString(data)

	// Set the response header and write the encoded image data to the response body
	w.Header().Set("Content-Type", "text/plain")
	w.Write([]byte(encoded))
}
func (rp *Repository) SalesGetCustomerList(w http.ResponseWriter, r *http.Request) {
	//rp.EnableCors(&w)
	sr := GenerateServerResponse("SalesCustomerList", GET)
	rp.ProcessSalesRequest(w, r, rp.DoSalesGetCustomerList, nil, sr)
}
func (rp *Repository) SalesGetCustomerRecordByKodeCust(w http.ResponseWriter, r *http.Request) {
	//rp.EnableCors(&w)
	sr := GenerateServerResponse("SalesCustomerRecord", GET)
	sKodeCust := chi.URLParam(r, "KodeCust")
	rp.ProcessSalesRequest(w, r, rp.DoSalesGetCustomerRecordByKodeCust, sKodeCust, sr)
}
func (rp *Repository) SalesCreateCustomerRecord(w http.ResponseWriter, r *http.Request) {
	//rp.EnableCors(&w)

	sr := GenerateServerResponse("SalesCustomerRecord", POST)
	rp.ProcessSalesRequestWithBinding(w, r, rp.DoSalesCreateCustomerRecord, &models.CustomerData{}, nil, sr)
}
func (rp *Repository) SalesUpdateCustomerRecord(w http.ResponseWriter, r *http.Request) {
	//rp.EnableCors(&w)
	sr := GenerateServerResponse("SalesCustomerRecord", PUT)
	rp.ProcessSalesRequestWithBinding(w, r, rp.DoSalesUpdateCustomerRecord, &models.CustomerData{}, nil, sr)
}

func (rp *Repository) SalesCheckIn(w http.ResponseWriter, r *http.Request) {
	sr := GenerateServerResponse("CheckIn", POST)
	rp.ProcessSalesRequestWithBinding(w, r, rp.DoSalesCheckIn, &models.SalesCheckIn{}, nil, sr)
}
func (rp *Repository) SalesCheckOut(w http.ResponseWriter, r *http.Request) {
	sr := GenerateServerResponse("CheckOut", POST)
	rp.ProcessSalesRequestWithBinding(w, r, rp.DoSalesCheckOut, &models.SalesCheckOut{}, nil, sr)
}
func (rp *Repository) SalesGetVisit(w http.ResponseWriter, r *http.Request) {
	sr := GenerateServerResponse("GetVisit", GET)
	rp.ProcessSalesRequest(w, r, rp.DoGetVisit, nil, sr)
}
func (rp *Repository) SalesGetProduct(w http.ResponseWriter, r *http.Request) {
	sr := GenerateServerResponse("GetProduct", GET)
	rp.ProcessSalesRequest(w, r, rp.DoGetProduct, nil, sr)
}
func (rp *Repository) SalesDeleteOrderByUID(w http.ResponseWriter, r *http.Request) {
	sr := GenerateServerResponse("DeleteOrder", DELETE)
	sUID := chi.URLParam(r, "UID")
	rp.ProcessSalesRequest(w, r, rp.DoDeleteOrder, sUID, sr)
}

func (rp *Repository) SalesDeletePaymentByUID(w http.ResponseWriter, r *http.Request) {
	sr := GenerateServerResponse("DeletePayment", DELETE)
	sUID := chi.URLParam(r, "UID")
	rp.ProcessSalesRequest(w, r, rp.DoDeletePayment, sUID, sr)
}

func (rp *Repository) SalesGetSalesOrderList(w http.ResponseWriter, r *http.Request) {
	//rp.EnableCors(&w)
	sr := GenerateServerResponse("SalesOrderList", GET)
	rp.ProcessSalesRequest(w, r, rp.DoSalesGetSalesOrderList, nil, sr)
}
func (rp *Repository) SalesGetSalesOrderRecordByNoOrder(w http.ResponseWriter, r *http.Request) {
	//rp.EnableCors(&w)
	sr := GenerateServerResponse("SalesOrderRecord", GET)
	sNoOrder := chi.URLParam(r, "NoOrder")
	rp.ProcessSalesRequest(w, r, rp.DoSalesGetSalesOrderRecord, sNoOrder, sr)
}
func (rp *Repository) SalesCreateSalesOrderRecord(w http.ResponseWriter, r *http.Request) {
	//rp.EnableCors(&w)
	sr := GenerateServerResponse("SalesOrderRecord", POST)
	rp.ProcessSalesRequestWithBinding(w, r, rp.DoSalesCreateSalesOrderRecord, &models.OrderData{}, nil, sr)
}
func (rp *Repository) SalesUpdateSalesOrderRecord(w http.ResponseWriter, r *http.Request) {
	//rp.EnableCors(&w)
	sr := GenerateServerResponse("SalesOrderRecord", PUT)
	rp.ProcessSalesRequestWithBinding(w, r, rp.DoSalesUpdateSalesOrderRecord, &models.OrderData{}, nil, sr)
}

func (rp *Repository) SalesGetPaymentList(w http.ResponseWriter, r *http.Request) {
	//rp.EnableCors(&w)
	sr := GenerateServerResponse("SalesPaymentList", GET)
	rp.ProcessSalesRequest(w, r, rp.DoSalesGetPaymentList, nil, sr)
}
func (rp *Repository) SalesGetPaymentRecordByNoTransaksi(w http.ResponseWriter, r *http.Request) {
	//rp.EnableCors(&w)
	sr := GenerateServerResponse("SalesPaymentRecord", GET)
	sNoTransaksi := chi.URLParam(r, "NoTransaksi")
	rp.ProcessSalesRequest(w, r, rp.DoSalesGetPaymentRecord, sNoTransaksi, sr)
}
func (rp *Repository) SalesCreatePaymentRecord(w http.ResponseWriter, r *http.Request) {
	//rp.EnableCors(&w)
	sr := GenerateServerResponse("SalesPaymentRecord", POST)
	rp.ProcessSalesRequestWithBinding(w, r, rp.DoSalesCreatePaymentRecord, &models.PaymentData{}, nil, sr)
}
func (rp *Repository) SalesUpdatePaymentRecord(w http.ResponseWriter, r *http.Request) {
	//rp.EnableCors(&w)
	sr := GenerateServerResponse("SalesPaymentRecord", PUT)
	rp.ProcessSalesRequestWithBinding(w, r, rp.DoSalesUpdatePaymentRecord, &models.PaymentData{}, nil, sr)
}
