package models

import (
	"github.com/mholt/binding"
	"github.com/shopspring/decimal"
	"math"
	"net/http"
)

type Location struct {
	Lat float64 `json:"lat"`
	Lng float64 `json:"lng"`
}

func (loc *Location) IsNull() bool {
	return math.Abs(loc.Lat) > 1 && math.Abs(loc.Lng) > 1
}

type CustomerData struct {
	UID               string
	KodePT            string
	KodeCust          string
	NamaCust          string
	AlamatCust        string
	GPSLocation       Location
	Kota              string
	Telp              string
	Kecamatan         string
	Pemilik           string
	NoKTP             string
	NamaFileFotoKTP   string
	NamaFileFotoToko1 string
	NamaFileFotoToko2 string
	NamaFileFotoToko3 string
	FileFotoKTP       string
	FileFotoToko1     string
	FileFotoToko2     string
	FileFotoToko3     string
	AlamatPemilik     string
	TipeCust          string
	CreditLimit       decimal.Decimal
	Piutang           decimal.Decimal
	JmlArmada         string
	LevelToko         string
	SistemPembayaran  string
	StatusKepemilikan string
	KodeSales         string
	NPWP              string
	Aktif             string
	FlagPiutang45     SQLBool
	FlagLebihPlafon   SQLBool
}

func (c *CustomerData) FieldMap(req *http.Request) binding.FieldMap {
	return binding.FieldMap{
		&c.KodePT: binding.Field{
			Required: false,
		},
		&c.KodeCust: binding.Field{
			Required:     true,
			ErrorMessage: "Kode Customer wajib diisi!",
		},
		&c.NamaCust: binding.Field{
			Required:     true,
			ErrorMessage: "Nama Customer wajib diisi!",
		},
		&c.AlamatCust: binding.Field{
			Required:     true,
			ErrorMessage: "Alamat Customer wajib diisi!",
		},
	}
}
