package models

import (
	"database/sql/driver"
	"encoding/json"
	"github.com/shopspring/decimal"
	"strings"
	"time"
)

type SQLDate time.Time
type SQLDateTime time.Time
type SQLTime time.Time

func (d *SQLDate) UnmarshalJSON(b []byte) error {
	s := strings.Trim(string(b), "\"")
	if s == "null" {
		*d = SQLDate(time.Time{})
		return nil
	}
	t, err := time.Parse("2006-01-02", s)
	if err != nil {
		return err
	}
	*d = SQLDate(t)
	return nil
}

func (d SQLDate) MarshalJSON() ([]byte, error) {
	if time.Time(d).IsZero() {
		return []byte("null"), nil
	} else {
		return json.Marshal(time.Time(d).Format("2006-01-02"))
	}
}

func (d SQLDate) Format() string {
	t := time.Time(d)
	return t.Format("2006-01-02")
}

func (d SQLDate) FromTime(t time.Time) time.Time {
	return t
}

func (d *SQLDate) Scan(value interface{}) error {
	// if value is nil, false
	if value == nil {
		return nil
	} else {
		*d = SQLDate(value.(time.Time))
		return nil
	}
}

func (d SQLDate) Value() (driver.Value, error) {
	if time.Time(d).IsZero() {

		return nil, nil
	}
	return time.Time(d).Format("2006-01-02"), nil
}

func (d *SQLDateTime) UnmarshalJSON(b []byte) error {
	s := strings.Trim(string(b), "\"")
	if s == "null" {
		*d = SQLDateTime(time.Time{})
		return nil
	}
	t, err := time.Parse("2006-01-02 15:04:05", s)
	if err != nil {
		return err
	}
	*d = SQLDateTime(t)
	return nil
}

func (d SQLDateTime) MarshalJSON() ([]byte, error) {
	return json.Marshal(time.Time(d).Format("2006-01-02 15:04:05"))
}

func (d SQLDateTime) Format() string {
	t := time.Time(d)
	return t.Format("2006-01-02 15:04:05")
}

func (d SQLDateTime) FromTime(t time.Time) time.Time {
	return t
}

func (d *SQLDateTime) Scan(value interface{}) error {
	// if value is nil, false
	if value == nil {
		return nil
	} else {
		*d = SQLDateTime(value.(time.Time))
		return nil
	}
}

func (d SQLDateTime) Value() (driver.Value, error) {
	if time.Time(d).IsZero() {

		return nil, nil
	}
	return time.Time(d).Format("2006-01-02 15:04:05"), nil
}

func (d *SQLTime) UnmarshalJSON(b []byte) error {
	s := strings.Trim(string(b), "\"")
	if s == "null" {
		*d = SQLTime(time.Time{})
		return nil
	}
	t, err := time.Parse("15-04-05", s)
	if err != nil {
		return err
	}
	*d = SQLTime(t)
	return nil
}

func (d SQLTime) MarshalJSON() ([]byte, error) {
	if time.Time(d).IsZero() {
		return []byte("null"), nil
	} else {
		return json.Marshal(time.Time(d).Format("15:04:05"))
	}
}

func (d SQLTime) Format() string {
	t := time.Time(d)
	return t.Format("15:04:05")
}

func (d SQLTime) FromTime(t time.Time) time.Time {
	return t
}

func (d *SQLTime) Scan(value interface{}) error {
	// if value is nil, false
	if value == nil {
		return nil
	} else {
		*d = SQLTime(value.(time.Time))
		return nil
	}
}

func (d SQLTime) Value() (driver.Value, error) {
	if time.Time(d).IsZero() {

		return nil, nil
	}
	return time.Time(d).Format("15:04:05"), nil
}

type TLoginState int

const (
	LoggedIn TLoginState = iota
	LoggedOut
)

type TResetPasswordStatus int

const (
	RPPending TResetPasswordStatus = iota
	RPApproved
	RPRejected
	RPCanceled
	RPError
)

func (ls TLoginState) String() string {
	switch ls {
	case LoggedIn:
		return "LoggedIn"
	case LoggedOut:
		return "LoggedOut"
	}
	return "unknown"
}
func (rps TResetPasswordStatus) String() string {
	switch rps {
	case RPPending:
		return "PendingApproval"
	case RPApproved:
		return "Approved"
	case RPRejected:
		return "Rejected"
	case RPCanceled:
		return "Canceled"
	}
	return "Error"
}

type SQLBool string

const (
	Ya    SQLBool = "Ya"
	Tidak         = "Tidak"
)

type ChangePasswordResponse struct {
	PasswordChanged bool
	LoginState      string
}
type ResetPasswordResponse struct {
	RequestSent   bool
	RequestID     string
	RequestStatus string
}

type TemplateData struct {
	StringMap map[string]string
	IntMap    map[string]int
	FloatMap  map[string]float32
	Data      map[string]interface{}
	CSRFToken string
	Flash     string
	Success   string
	Warning   string
	Err       string
}

type GeneralServerResponse struct {
	Message            string      `json:"message,omitempty"`
	ResponseData       interface{} `json:"responseData,omitempty"`
	ResponseDataLength *int        `json:"responseDataLength,omitempty"`
	Error              bool        `json:"error"`
	HTTPStatusCode     int         `json:"HTTPStatusCode"`
	DebugMessage       string      `json:"debugMessage,omitempty"`
	DebugErrMessage    string      `json:"DebugErrMessage,omitempty"`
	DebugData          interface{} `json:"debugData,omitempty"`
	ErrorMessage       string      `json:"errorMessage,omitempty"`
}

type RKSData struct {
	KodePT         string
	KodeSales      string
	Hari           string
	KodeCust       string
	NamaCust       string
	CreditLimit    decimal.Decimal
	SisaCredit     decimal.Decimal
	TotalPiutang   decimal.Decimal
	PiutangLebih45 decimal.Decimal
	CustomerData   CustomerData
}
