package models

import (
	"github.com/mholt/binding"
	"github.com/shopspring/decimal"
	"net/http"
)

type ApprovalStatus string

const (
	Pending  ApprovalStatus = "Pending"
	Approved                = "Approved"
	Canceled                = "Canceled"
	NoChange                = "NoChange"
)

//func (as ApprovalStatus) String() string {
//	switch as {
//	case SOPending:
//		return "Pending"
//	case SOApproved:
//		return "Approved"
//	case SOCanceled:
//		return "Canceled"
//	}
//	return "unknown"
//}

type TipeBarang string

const (
	ReadyStock TipeBarang = "System"
	NewRequest            = "NewRequest"
)

type TipeDiskon string

const (
	DiskonPersen TipeDiskon = "Persen"
	DiskonRupiah            = "Rupiah"
)

type Product struct {
	KodeBrg string
	NamaBrg string
	Satuan  string
	Stock   decimal.Decimal
	Konv    decimal.Decimal
}
type OrderItems struct {
	NoUrut       int
	KodeBrg      string
	NamaBrg      string
	Satuan       string
	Konv         decimal.Decimal
	Jml          decimal.Decimal
	Harga        decimal.Decimal
	DiskonPersen decimal.Decimal
	DiskonRupiah decimal.Decimal
	TipeDiskon   TipeDiskon
	HargaDiskon  decimal.Decimal
	Total        decimal.Decimal
	TipeBrg      TipeBarang
}

func (od OrderItems) FieldMap(request *http.Request) binding.FieldMap {
	return binding.FieldMap{
		&od.KodeBrg: binding.Field{
			Required: true,
		},
		&od.Jml: binding.Field{
			Required: true,
		},
	}
}

type OrderData struct {
	UID            string
	KodePT         string
	KodeSales      string
	NoOrder        string
	NoSO           string
	KodeCust       string
	NamaCust       string
	TglSO          SQLDate
	TglJT          SQLDate
	Nilai          decimal.Decimal
	Keterangan     string
	Items          []OrderItems
	ApprovalStatus ApprovalStatus
}

func (om OrderData) FieldMap(request *http.Request) binding.FieldMap {
	return binding.FieldMap{
		&om.KodeCust: binding.Field{Required: true},
		&om.TglSO: binding.Field{
			Required:   true,
			TimeFormat: "2006-01-02",
		},
	}
}
