package models

import (
	"github.com/mholt/binding"
	"net/http"
)

type LoginCredential struct {
	NoHP     string
	Username string
	Password string
}
type LogoutStatus struct {
	Login string
	Msg   string
}

func (lc *LoginCredential) FieldMap(req *http.Request) binding.FieldMap {
	return binding.FieldMap{
		&lc.NoHP: binding.Field{
			Required: false,
			//ErrorMessage: "Nomor Handphone wajib diisi!",
		},
		&lc.Username: binding.Field{
			Required:     true,
			ErrorMessage: "Username wajib diisi!",
		},
		&lc.Password: binding.Field{
			Required:     true,
			ErrorMessage: "Password wajib diisi!",
		},
	}
}

type ChangeCredential struct {
	KodePT      string
	Username    string
	OldPassword string
	NewPassword string
}

func (cc *ChangeCredential) FieldMap(req *http.Request) binding.FieldMap {
	return binding.FieldMap{
		&cc.KodePT: binding.Field{
			Required: false,
		},
		&cc.Username: binding.Field{
			Form:         "username",
			Required:     true,
			ErrorMessage: "Username wajib diisi!",
		},
		&cc.OldPassword: binding.Field{
			Form:         "oldpassword",
			Required:     true,
			ErrorMessage: "Password wajib diisi!",
		},
		&cc.NewPassword: binding.Field{
			Form:         "newpassword",
			Required:     true,
			ErrorMessage: "Password baru wajib diisi!",
		},
	}
}

type SalesData struct {
	KodeSales   string
	KodePT      string
	Password    string
	NoHandphone string
	NamaSales   string
	Area        string
	Aktif       SQLBool
}

type SalesLoginResponse struct {
	Authenticated bool
	Sales         SalesData
	AuthString    string
}

type SalesCheckIn struct {
	UID              string
	KodeSales        string
	KodePT           string
	NamaCust         string
	CheckInLoc       Location
	CheckIn          SQLDateTime
	CheckInDalamArea SQLBool
	KodeCust         string
}

func (cc *SalesCheckIn) FieldMap(req *http.Request) binding.FieldMap {
	return binding.FieldMap{
		&cc.UID: binding.Field{
			Required:     true,
			ErrorMessage: "UID wajib diisi!",
		},
		&cc.KodeSales: binding.Field{
			Required: false,
		},
		&cc.KodePT: binding.Field{
			Required: false,
		},

		&cc.NamaCust: binding.Field{
			Required: false,
		},
		&cc.CheckInLoc: binding.Field{
			Required: true,
		},
		&cc.CheckIn: binding.Field{
			Required: true,
		},
		&cc.CheckInDalamArea: binding.Field{
			Required: true,
		},
		&cc.KodeCust: binding.Field{
			Required: true,
		},
	}
}

type SalesCheckOut struct {
	UID               string
	KodeSales         string
	KodePT            string
	CheckOutLoc       Location
	CheckOut          SQLDateTime
	KodeCust          string
	CheckOutDalamArea SQLBool
}

func (cc *SalesCheckOut) FieldMap(req *http.Request) binding.FieldMap {
	return binding.FieldMap{
		&cc.UID: binding.Field{
			Required:     true,
			ErrorMessage: "UID wajib diisi!",
		},
		&cc.KodeSales: binding.Field{
			Required: false,
		},
		&cc.KodePT: binding.Field{
			Required: false,
		},
		&cc.CheckOutLoc: binding.Field{
			Required: true,
		},
		&cc.CheckOut: binding.Field{
			Required: true,
		},
		&cc.CheckOutDalamArea: binding.Field{
			Required: true,
		},
		&cc.KodeCust: binding.Field{
			Required: true,
		},
	}
}
