package render

import (
	"github.com/justinas/nosurf"
	config "github.com/violendy/SalesPortal/internal/config"
	models "github.com/violendy/SalesPortal/internal/models"
	"html/template"
	"log"
	"net/http"
	"path/filepath"
)

var functions = template.FuncMap{}
var app *config.AppConfig

func NewTemplates(a *config.AppConfig) {
	app = a
}

func AddDefaultData(td *models.TemplateData, r *http.Request) *models.TemplateData {
	td.CSRFToken = nosurf.Token(r)
	return td
}

func Template(w http.ResponseWriter, templateName string, r *http.Request, td *models.TemplateData) {
	var err error
	var TemplateCaches map[string]*template.Template

	if app.UseCache {
		TemplateCaches = app.TemplateCache
	} else {
		TemplateCaches, err = CreateTemplateCache()
		if err != nil {
			log.Fatal("Error reading templates from disk")
		}
	}

	RenderedTemplate, ok := TemplateCaches[templateName+".page.gohtml"]
	if !ok {
		log.Fatal("Template don't exists!")
	}

	td = AddDefaultData(td, r)
	err = RenderedTemplate.Execute(w, td)

	if err != nil {
		log.Println(err)
		return
	}

}

func CreateTemplateCache() (map[string]*template.Template, error) {

	TemplateCaches := map[string]*template.Template{}
	var allPages []string

	pages, err := filepath.Glob("./templates/*.page.gohtml")
	if err != nil {
		log.Fatal("Template file list empty!")
	}
	allPages = append(allPages, pages...)

	for _, page := range allPages {
		templateName := filepath.Base(page) // strings.Replace(page, "templates\\", "", -1 )

		TemplateSet, err := template.New(templateName).Funcs(functions).ParseFiles(page)
		if err != nil {
			log.Fatal(err)
		}
		layoutExists, err := filepath.Glob("./templates/*.layout.gohtml")
		if err != nil {
			log.Fatal(err)
		}
		if len(layoutExists) > 0 {
			TemplateSet, err = TemplateSet.ParseGlob("./templates/*.layout.gohtml")
			if err != nil {
				log.Fatal(err)
			}
		}
		TemplateCaches[templateName] = TemplateSet
	}
	return TemplateCaches, nil
}
