package apirepo

import (
	"context"
	"fmt"
	_ "github.com/mattn/go-sqlite3"
	"github.com/mdp/qrterminal"
	"go.mau.fi/whatsmeow"
	waProto "go.mau.fi/whatsmeow/binary/proto"
	"go.mau.fi/whatsmeow/store/sqlstore"
	"go.mau.fi/whatsmeow/types"
	waLog "go.mau.fi/whatsmeow/util/log"
	"google.golang.org/protobuf/proto"
	"log"
	"os"
	"regexp"
	"strings"
	"time"
)

func WAConnect() (*whatsmeow.Client, error) {
	container, err := sqlstore.New("sqlite3", "file:wapp.db?_foreign_keys=on", waLog.Noop)
	if err != nil {
		return nil, err
	}
	deviceStore, err := container.GetFirstDevice()
	if err != nil {
		panic(err)
	}
	client := whatsmeow.NewClient(deviceStore, waLog.Noop)
	if client.Store.ID == nil {
		// No ID stored, new login
		qrChan, _ := client.GetQRChannel(context.Background())
		err = client.Connect()
		if err != nil {
			return nil, err
		}
		for evt := range qrChan {
			if evt.Event == "code" {
				qrterminal.GenerateHalfBlock(evt.Code, qrterminal.L, os.Stdout)
			} else {
				fmt.Println("Login event:", evt.Event)
			}
		}
	} else {
		err := client.Connect()
		if err != nil {
			return nil, err
		}
	}
	return client, nil
}

func (rp *WARepo) FormatPhoneToJID(sNoHP string) string {
	input := strings.TrimSpace(sNoHP)

	reg := regexp.MustCompile("[^0-9]+")
	output := reg.ReplaceAllString(input, "")

	if strings.HasPrefix(output, "0") {
		output = "62" + output[1:]
	}
	if strings.HasPrefix(output, "8") {
		output = "62" + output
	}

	return output
}
func (rp *WARepo) SendMessage(sNoHP string, sMessage string) error {

	ctx, cancel := context.WithTimeout(context.Background(), 15*time.Minute)
	defer cancel()
	wac, err := WAConnect()
	if err != nil {
		log.Println(err)
		return err
	}
	defer wac.Disconnect()
	targetJID := types.NewJID(rp.FormatPhoneToJID(sNoHP), types.DefaultUserServer)
	msg := &waProto.Message{
		Conversation: proto.String(sMessage),
	}
	_, err = wac.SendMessage(ctx, targetJID, msg)
	if err != nil {
		log.Println(err)
		return err
	}
	return nil
	//
	//data := url.Values{}
	//data.Set("token", "MqE6KKxzE8zySr6S4rmzbcj2rSAKNk3tYiBonBm9GgoxrNfMeU")
	//data.Set("number", sNoHP)
	//data.Set("message", sMessage)
	//data.Set("date", time.Now().Format("2006-01-02"))
	//data.Set("time", time.Now().Add(30*time.Second).Format("15:04:05"))
	//uri := "https://app.ruangwa.id/api/send_message"
	//encodedData := data.Encode()
	//req, err := http.NewRequest("POST", uri, strings.NewReader(encodedData))
	//if err != nil {
	//	log.Println(err)
	//	return err
	//}
	//req.Header.Set("Content-Type", "application/x-www-form-urlencoded")
	//
	//client := &http.Client{}
	//resp, err := client.Do(req)
	//if err != nil {
	//	log.Println(err)
	//	return err
	//}
	//defer resp.Body.Close()
	//log.Println("OK", resp.Body)
	//log.Println("Response status:", resp.Status)
	//return nil
}
