package dbrepo

import (
	"context"
	"errors"
	"github.com/violendy/SalesPortal/internal/models"
	"log"
)

func (rp *nkDBRepo) GetSyncSales(sKodePT string, sTransID string, sSales string) (int, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()

	qry := `SELECT GetSyncSales(?,?)`
	row, err := rp.DB[sKodePT].QueryContext(ctx, qry, sTransID, sSales)
	if err != nil {
		log.Println(err)
		return -1, err
	}
	var sID int

	for row.Next() {
		err = row.Scan(&sID)
		if err != nil {
			log.Println(err)
			return -1, err
		}
	}
	return sID, nil
}
func (rp *nkDBRepo) RKSListGet(sKodePT string) ([]models.RKSData, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()
	qry := `CALL dbRKSListGet()`
	row, err := rp.DB[sKodePT].QueryContext(ctx, qry)
	if err != nil {
		log.Println(err)
		return []models.RKSData{}, err
	}

	var RKSData []models.RKSData

	defer func() {
		err := row.Close()
		if err != nil {
			log.Println("Error closing rows!")
		}
	}()
	for row.Next() {
		var RKS models.RKSData
		err = row.Scan(&RKS.KodeSales, &RKS.Hari, &RKS.KodeCust, &RKS.NamaCust, &RKS.CreditLimit, &RKS.SisaCredit, &RKS.TotalPiutang, &RKS.PiutangLebih45)
		if err != nil {
			log.Println(err)
			return []models.RKSData{}, err
		}
		RKS.KodePT = sKodePT
		RKSData = append(RKSData, RKS)
	}
	return RKSData, nil
}
func (rp *nkDBRepo) RKSListGetBySales(sKodePT string, sKodeSales string) ([]models.RKSData, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()
	qry := `CALL dbRKSListGet(NOW(),? )`
	row, err := rp.DB[sKodePT].QueryContext(ctx, qry, sKodeSales)
	if err != nil {
		log.Println(err)
		return []models.RKSData{}, err
	}

	var RKSData []models.RKSData

	defer func() {
		err := row.Close()
		if err != nil {
			log.Println("Error closing rows!")
		}
	}()
	for row.Next() {
		var RKS models.RKSData
		err = row.Scan(&RKS.KodeSales, &RKS.Hari, &RKS.KodeCust, &RKS.NamaCust, &RKS.CreditLimit, &RKS.SisaCredit, &RKS.TotalPiutang, &RKS.PiutangLebih45)
		if err != nil {
			log.Println(err)
			return []models.RKSData{}, err
		}
		RKS.KodePT = sKodePT
		RKSData = append(RKSData, RKS)
	}
	return RKSData, nil
}
func (rp *nkDBRepo) CustomerListGet(sKodePT string) ([]models.CustomerData, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()
	qry := `CALL GetAllCustomerList()`
	row, err := rp.DB[sKodePT].QueryContext(ctx, qry)
	if err != nil {
		log.Println(err)
		return []models.CustomerData{}, err
	}

	var CustomerData []models.CustomerData

	defer func() {
		err := row.Close()
		if err != nil {
			log.Println("Error closing rows!")
		}
	}()
	for row.Next() {
		var Customer models.CustomerData
		err = row.Scan(
			&Customer.KodeCust,
			&Customer.NamaCust,
			&Customer.AlamatCust,
			&Customer.GPSLocation.Lat,
			&Customer.GPSLocation.Lng,
			&Customer.Kota,
			&Customer.Telp,
			&Customer.Kecamatan,
			&Customer.Pemilik,
			&Customer.NoKTP,
			&Customer.FileFotoKTP,
			&Customer.FileFotoToko1,
			&Customer.FileFotoToko2,
			&Customer.AlamatPemilik,
			&Customer.TipeCust,
			&Customer.CreditLimit,
			&Customer.Piutang,
			&Customer.JmlArmada,
			&Customer.LevelToko,
			&Customer.SistemPembayaran,
			&Customer.StatusKepemilikan,
			&Customer.KodeSales,
			&Customer.NPWP,
			&Customer.Aktif,
		)
		if err != nil {
			log.Println(err)
			return []models.CustomerData{}, err
		}
		Customer.KodePT = sKodePT
		CustomerData = append(CustomerData, Customer)
	}
	return CustomerData, nil
}
func (rp *nkDBRepo) SalesGetProduct(sKodePT string) (*[]models.Product, int, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()
	qry := `SELECT a.KodeBrg, a.NamaBrg, SUM(IFNULL(b.StockAkhir,0)), a.Satuan, 1 FROM tm_barang a LEFT JOIN tm_stockbrg b ON b.KodeBrg=a.KodeBrg GROUP BY KodeBrg`
	row, err := rp.DB[sKodePT].QueryContext(ctx, qry)
	if err != nil {
		log.Println(err)
		return nil, 0, err
	}
	var ProductList []models.Product
	for row.Next() {
		var ProductRow models.Product
		err = row.Scan(
			&ProductRow.KodeBrg,
			&ProductRow.NamaBrg,
			&ProductRow.Stock,
			&ProductRow.Satuan,
			&ProductRow.Konv,
		)
		if err != nil {
			log.Println(err)
			return nil, 0, err
		}
		ProductList = append(ProductList, ProductRow)
	}
	return &ProductList, len(ProductList), nil
}
func (rp *nkDBRepo) CustomerListGetBySales(sKodePT string, sKodeSales string) ([]models.CustomerData, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()
	qry := `CALL GetAllCustomerListBySales(?)`
	row, err := rp.DB[sKodePT].QueryContext(ctx, qry, sKodeSales)
	if err != nil {
		log.Println(err)
		return []models.CustomerData{}, err
	}

	var CustomerData []models.CustomerData

	defer func() {
		err := row.Close()
		if err != nil {
			log.Println("Error closing rows!")
		}
	}()
	for row.Next() {
		var Customer models.CustomerData
		err = row.Scan(
			&Customer.KodeCust,
			&Customer.NamaCust,
			&Customer.AlamatCust,
			&Customer.GPSLocation.Lat,
			&Customer.GPSLocation.Lng,
			&Customer.Kota,
			&Customer.Telp,
			&Customer.Kecamatan,
			&Customer.Pemilik,
			&Customer.NoKTP,
			&Customer.FileFotoKTP,
			&Customer.FileFotoToko1,
			&Customer.FileFotoToko2,
			&Customer.AlamatPemilik,
			&Customer.TipeCust,
			&Customer.CreditLimit,
			&Customer.Piutang,
			&Customer.JmlArmada,
			&Customer.LevelToko,
			&Customer.SistemPembayaran,
			&Customer.StatusKepemilikan,
			&Customer.KodeSales,
			&Customer.NPWP,
			&Customer.FlagPiutang45,
			&Customer.FlagLebihPlafon,
			&Customer.Aktif,
		)
		if err != nil {
			log.Println(err)
			return []models.CustomerData{}, err
		}
		Customer.KodePT = sKodePT
		CustomerData = append(CustomerData, Customer)
	}
	return CustomerData, nil
}
func (rp *nkDBRepo) CustomerGetByKodeCust(sKodePT string, sKodeCust string) (*models.CustomerData, error) {
	return nil, nil
}

func (rp *nkDBRepo) CustomerAdd(sKodePT string, data models.CustomerData) error {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()

	var Result string
	qry := `CALL dbCreateNewCustomer(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)`
	err := rp.DB[sKodePT].QueryRowContext(ctx, qry,

		&data.KodeCust,
		&data.NamaCust,
		&data.AlamatCust,
		&data.Kota,
		&data.Telp,
		&data.Kecamatan,
		&data.Pemilik,
		&data.NoKTP,
		&data.AlamatPemilik,
		&data.TipeCust,
		&data.CreditLimit,
		&data.Piutang,
		&data.JmlArmada,
		&data.LevelToko,
		&data.SistemPembayaran,
		&data.StatusKepemilikan,
		&data.KodeSales,
		&data.NPWP,
		&data.Aktif).Scan(&Result)
	if err != nil {
		return err
	}
	if Result == "Record Saved" {
		return nil
	} else if Result == "Record Exists" {
		return errors.New("record already exists")
	} else {
		return errors.New("insert failed")
	}

}
func (rp *nkDBRepo) PaymentAdd(sKodePT string, data models.PaymentData) error {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()

	if data.UID == "" {
		qry := `SELECT UUID()`
		var sUID string
		err := rp.DB[sKodePT].QueryRowContext(ctx, qry).Scan(&sUID)
		if err != nil {
			log.Println("error generating payment UUID")
		}
		data.UID = sUID
	}
	if data.NoBayar == "" {
		var sNoBayar string
		qry := `SELECT BuatNo('Tr_BayarJual','YJ',DATE_FORMAT(?,'%Y-%m-%d'),'YJ','07000001')`
		err := rp.DB[sKodePT].QueryRowContext(ctx, qry, data.TglBayar).Scan(&sNoBayar)
		if err != nil {
			return err
		}
		data.NoBayar = sNoBayar
	}

	var Result string
	for _, item := range data.Items {

		qry := `CALL dbCreateNewPayment(?,?,?,?,?,?,?,?,?,?,?)`
		err := rp.DB[sKodePT].QueryRowContext(ctx, qry,
			data.UID,
			data.NoBayar,
			data.KodeCust,
			data.NoJual,
			data.TglBayar,
			data.Nilai,
			item.NoUrut,
			item.Nilai,
			item.KodeBayar,
			item.NoBG,
			item.Bank,
		).Scan(&Result)
		if err != nil {
			return err
		}
	}
	if Result == "Record Saved" {
		return nil
	} else {
		return errors.New("insert failed")
	}

}

//func (rp *nkDBRepo) GetAllSalesOrderBySales(sKodePT string, sKodeSales string) ([]models.OrderData, error) {
//	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
//	defer cancel()
//	qry := `CALL GetAllCustomerListBySales(?)`
//	row, err := rp.DB[sKodePT].SQL.QueryContext(ctx, qry, sKodeSales)
//	if err != nil {
//		log.Println(err)
//		return []models.CustomerData{}, err
//	}
//
//	var CustomerData []models.CustomerData
//
//	defer func() {
//		err := row.Close()
//		if err != nil {
//			log.Println("Error closing rows!")
//		}
//	}()
//	for row.Next() {
//		var Customer models.CustomerData
//		err = row.Scan(
//			&Customer.KodeCust,
//			&Customer.NamaCust,
//			&Customer.AlamatCust,
//			&Customer.GPSLocation.Lat,
//			&Customer.GPSLocation.Lng,
//			&Customer.Kota,
//			&Customer.Telp,
//			&Customer.Kecamatan,
//			&Customer.Pemilik,
//			&Customer.NoKTP,
//			&Customer.FileFotoKTP,
//			&Customer.FileFotoToko1,
//			&Customer.FileFotoToko2,
//			&Customer.AlamatPemilik,
//			&Customer.TipeCust,
//			&Customer.CreditLimitGalvalum,
//			&Customer.CreditLimitNonGalvalum,
//			&Customer.PiutangGalvalum,
//			&Customer.PiutangNonGalvalum,
//			&Customer.JmlArmada,
//			&Customer.LevelToko,
//			&Customer.SistemPembayaran,
//			&Customer.StatusKepemilikan,
//			&Customer.KodeSalesGalvalum,
//			&Customer.KodeSalesNonGalvalum,
//			&Customer.NPWP,
//			&Customer.CustomerMutasi,
//			&Customer.Aktif,
//		)
//		if err != nil {
//			log.Println(err)
//			return []models.CustomerData{}, err
//		}
//		Customer.KodePT = sKodePT
//		CustomerData = append(CustomerData, Customer)
//	}
//	return CustomerData, nil
//}
