package dbrepo

//TODO make sure all procedure with row has defer row.close()

import (
	"context"
	"crypto/md5"
	_ "database/sql"
	"encoding/base64"
	"errors"
	"fmt"
	"github.com/google/uuid"
	"github.com/violendy/SalesPortal/internal/models"
	"log"
	"math"
	"math/rand"
	"os"
	"path"
	"path/filepath"
	"time"
)

func (rp *portalDBRepo) CheckDalamArea(CustomerLocation models.Location, SalesLocation models.Location, radius float64) bool {

	CustomerLocation.Lat = CustomerLocation.Lat * math.Pi / 180
	CustomerLocation.Lng = CustomerLocation.Lng * math.Pi / 180
	SalesLocation.Lat = SalesLocation.Lat * math.Pi / 180
	SalesLocation.Lng = SalesLocation.Lng * math.Pi / 180

	var R float64
	R = 6371000

	a := math.Sin((SalesLocation.Lat-CustomerLocation.Lat)/2)*math.Sin((SalesLocation.Lat-CustomerLocation.Lat)/2) +
		math.Cos(CustomerLocation.Lat)*math.Cos(SalesLocation.Lat)*
			math.Sin((SalesLocation.Lng-CustomerLocation.Lng)/2)*math.Sin((SalesLocation.Lng-CustomerLocation.Lng)/2)
	c := 2 * math.Atan2(math.Sqrt(a), math.Sqrt(1-a))
	jarak := R * c

	return jarak <= radius
}
func (rp *portalDBRepo) NoBaruWithUID(TransID string, Tgl time.Time) (string, string, error) {
	ctx, cancel := context.WithTimeout(context.Background(), 5*rp.App.ContextTime)
	defer cancel()
	qry := `SELECT UUID()`
	row := rp.DB.QueryRow(qry)
	var sUID string
	err := row.Scan(&sUID)
	if err != nil {
		return "", "", errors.New("error generating UUID()")
	} else {
		qry = `SELECT dbNoBaruUID(?,?,?,?,?)`
		var sNomor string
		row = rp.DB.QueryRowContext(ctx, qry, sUID, TransID, time.Now().Format("2006-02-01"), 2, "Sales Order")
		err = row.Scan(&sNomor)
		if err != nil {
			return "", "", errors.New("error generating transaction no")
		} else {

			return sUID, sNomor, nil
		}
	}

}
func (rp *portalDBRepo) GetFile(md5checksum string) (string, error) {
	ctx, cancel := context.WithTimeout(context.Background(), 5*time.Minute)
	defer cancel()

	qry := `SELECT stored_name FROM t_files WHERE id=? LIMIT 1 `
	row, err := rp.DB.QueryContext(ctx, qry, md5checksum)
	var Result string
	if err != nil {
		return Result, err
	}
	foundFile := false
	for row.Next() {

		var storedName string
		err = row.Scan(&storedName)
		if err != nil {
			panic(err)
		}
		foundFile = true
		Result = storedName
	}
	if !foundFile {
		return Result, errors.New("file not found")
	}

	defer func() {
		err := row.Close()
		if err != nil {
			log.Println("Error closing rows!")
		}
	}()
	return Result, nil
}
func (rp *portalDBRepo) NewFile(filename string, md5checksum string) (string, error) {
	ctx, cancel := context.WithTimeout(context.Background(), 5*time.Minute)
	defer cancel()
	ext := path.Ext(filename)
	qry := `INSERT INTO t_files(id,filename, stored_name) VALUES(?,?,CONCAT(MD5(MD5(?)),?)) ON DUPLICATE KEY UPDATE filename=?`
	_, err := rp.DB.ExecContext(ctx, qry, md5checksum, filename, md5checksum, ext, filename)
	if err != nil {
		log.Println(err)
		return "", err
	}
	storedFile, err := rp.GetFile(md5checksum)
	if err != nil {
		log.Println(err)
		return "", err
	}
	return storedFile, nil
}

func (rp *portalDBRepo) UserLogin(credential *models.LoginCredential) (models.UserLoginResponse, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()

	qry := `SELECT dbUserLogin(?,?,?)`
	row, err := rp.DB.QueryContext(ctx, qry, credential.NoHP, credential.Username, credential.Password)
	if err != nil {
		log.Println(err)
		return models.UserLoginResponse{Authenticated: false, AuthString: "DENIED"}, err
	}
	var Token string
	for row.Next() {
		err = row.Scan(&Token)
		if err != nil {
			log.Println(err)
			return models.UserLoginResponse{Authenticated: false, AuthString: "DENIED"}, err
		}
	}
	if (Token != "DENIED") && (len(Token) == 40) {
		UserByToken, err := rp.UserGetByToken(Token)
		if err != nil {
			return models.UserLoginResponse{Authenticated: false, User: models.UserData{}, AuthString: Token}, errors.New("user dengan token sesuai request tidak ditemukan")
		} else {
			return models.UserLoginResponse{Authenticated: true, User: UserByToken, AuthString: Token}, nil
		}
	} else {
		return models.UserLoginResponse{Authenticated: false, User: models.UserData{}, AuthString: Token}, errors.New("bad username / password")
	}
}
func (rp *portalDBRepo) UserLogout(sToken string) (bool, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()
	qry := `SELECT dbUserLogout(?)`

	row, err := rp.DB.QueryContext(ctx, qry, sToken)
	if err != nil {
		log.Println(err)
		return false, err
	}
	var Result string
	for row.Next() {
		err = row.Scan(&Result)
		if err != nil {
			log.Println(err)
			return false, err
		}
	}
	return Result == "LoggedOut", nil
}
func (rp *portalDBRepo) UserGetByToken(sToken string) (models.UserData, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()

	qry := `CALL dbUserGetByToken(?)`
	var User models.UserData
	err := rp.DB.QueryRowContext(ctx, qry, sToken).Scan(&User.KodePT, &User.UserID, &User.UserName, &User.NoHandphone, &User.HakAkses, &User.LastLogin)
	if err != nil {
		return models.UserData{}, nil
	}
	return User, nil
}
func (rp *portalDBRepo) UserCheckToken(sToken string) bool {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()

	qry := `SELECT dbUserIsTokenValid(?)`
	row, err := rp.DB.QueryContext(ctx, qry, sToken)
	if err != nil {
		log.Println(err)
		return false
	}
	var Token string
	for row.Next() {
		err = row.Scan(&Token)
		if err != nil {
			log.Println(err)
			return false
		}
	}
	return Token == "TRUE"
}

func (rp *portalDBRepo) SalesLogin(credential *models.LoginCredential) (models.SalesLoginResponse, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()

	qry := `SELECT dbSalesLogin(?,?,?)`
	row, err := rp.DB.QueryContext(ctx, qry, credential.NoHP, credential.Username, credential.Password)
	if err != nil {
		log.Println(err)
		return models.SalesLoginResponse{Authenticated: false, AuthString: "DENIED"}, err
	}
	var Token string
	for row.Next() {
		err = row.Scan(&Token)
		if err != nil {
			log.Println(err)
			return models.SalesLoginResponse{Authenticated: false, AuthString: "DENIED"}, err
		}
	}
	if (Token != "DENIED") && (len(Token) == 40) {
		SalesByToken, err := rp.SalesGetByToken(Token)
		if err != nil {
			return models.SalesLoginResponse{Authenticated: false, Sales: models.SalesData{}, AuthString: Token}, errors.New("user dengan token sesuai request tidak ditemukan")
		} else {
			return models.SalesLoginResponse{Authenticated: true, Sales: SalesByToken, AuthString: Token}, nil
		}
	} else {
		return models.SalesLoginResponse{Authenticated: false, Sales: models.SalesData{}, AuthString: Token}, errors.New("bad username / password")
	}
}
func (rp *portalDBRepo) SalesLogout(sToken string) (bool, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()
	qry := `SELECT dbSalesLogout(?)`

	row, err := rp.DB.QueryContext(ctx, qry, sToken)
	if err != nil {
		log.Println(err)
		return false, err
	}
	var Result string
	for row.Next() {
		err = row.Scan(&Result)
		if err != nil {
			log.Println(err)
			return false, err
		}
	}
	return Result == "LoggedOut", nil
}
func (rp *portalDBRepo) SalesAdd(sKodePT string, Sales models.SalesData) error {
	//TODO create dbSalesAdd database procedure
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()
	var Result string
	qry := `CALL SalesAdd(?,?,?,?,?,?)`
	err := rp.DB.QueryRowContext(ctx, qry, Sales.KodePT, Sales.KodeSales, Sales.NamaSales, Sales.Password, Sales.Area, Sales.NoHandphone).Scan(&Result)
	if err != nil {
		return err
	}
	if Result == "RecordSaved" {
		return nil
	} else {
		return errors.New("record already exists")
	}
}
func (rp *portalDBRepo) SalesUpdate(sKodePT string, Sales models.SalesData) error {
	//TODO create dbSalesUpdate database procedure
	//TODO pastikan query sales mempertimbangkan status aktif

	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()
	var Result string
	qry := `CALL SalesUpdate(?,?,?,?,?,?)`
	err := rp.DB.QueryRowContext(ctx, qry, Sales.KodePT, Sales.KodeSales, Sales.NamaSales, Sales.Area, Sales.NoHandphone, Sales.Aktif).Scan(&Result)
	if err != nil {
		return err
	}
	if Result == "RecordSaved" {
		return nil
	} else {
		return errors.New("update failed")
	}
}
func (rp *portalDBRepo) SalesListGet(sKodePT string) (*[]models.SalesData, int, error) {
	// TODO create SalesListGet db procedure
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()

	qry := `CALL SalesListGet(?)`
	row, err := rp.DB.QueryContext(ctx, qry, sKodePT)
	if err != nil {

	}
	defer row.Close()
	var SalesList []models.SalesData
	for row.Next() {
		var Sales models.SalesData
		err := row.Scan(&Sales.KodePT, &Sales.KodeSales, &Sales.NamaSales, &Sales.Area, &Sales.NoHandphone, &Sales.Aktif)
		if err != nil {
			return nil, 0, err
		}
		SalesList = append(SalesList, Sales)
	}
	return &SalesList, len(SalesList), nil
}

func (rp *portalDBRepo) SalesGetByToken(sToken string) (models.SalesData, error) {
	ctx, cancel := context.WithTimeout(context.Background(), 5*rp.App.ContextTime)
	defer cancel()
	qry := `CALL dbSalesGetByToken(?)`
	row, err := rp.DB.QueryContext(ctx, qry, sToken)
	if err != nil {
		log.Println(err)
		return models.SalesData{}, err
	}
	var Sales models.SalesData

	for row.Next() {
		err = row.Scan(&Sales.KodePT, &Sales.KodeSales, &Sales.NamaSales, &Sales.NoHandphone, &Sales.Area)
		if err != nil {
			log.Println(err)
			return models.SalesData{}, err
		}
	}
	return Sales, nil
}
func (rp *portalDBRepo) SalesCheckToken(sToken string) bool {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()

	qry := `SELECT dbSalesIsTokenValid(?)`
	row, err := rp.DB.QueryContext(ctx, qry, sToken)
	if err != nil {
		log.Println(err)
		return false
	}
	var Token string
	for row.Next() {
		err = row.Scan(&Token)
		if err != nil {
			log.Println(err)
			return false
		}
	}
	return Token == "TRUE"
}
func (rp *portalDBRepo) SalesGetCustomerInArea(sKodePT string, sKodeSales string, sCurrentLocation models.Location) (error, int, *[]models.CustomerData) {
	CustomerList, err := rp.CustomerListGetBySales(sKodePT, sKodeSales)
	if err != nil {
		return err, 0, nil
	}
	var PossibleCustomer []models.CustomerData
	for _, Customer := range *CustomerList {
		if !Customer.GPSLocation.IsNull() {
			if rp.CheckDalamArea(Customer.GPSLocation, sCurrentLocation, 100) {
				PossibleCustomer = append(PossibleCustomer, Customer)
			}
		}
	}
	return nil, len(PossibleCustomer), &PossibleCustomer
}
func (rp *portalDBRepo) SalesGetVisit(sKodePT string, sKodeSales string) (*[]models.SalesVisit, int, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()

	qry := `SELECT 
				UID,
				KodePT,
				KodeSales,
				KodeCust,
				NamaCust,
				TglEntry,
				CheckIn,
				CheckOut,
				CheckInLat,
				CheckInLng,
				IFNULL(Lama,""),
				CheckOutLat,
				CheckOutLng,
				IFNULL(CheckInOut,"Tidak"),
				IFNULL(CheckInDalamArea,"Tidak"),
				IFNULL(CheckOutDalamArea,"Tidak")
			FROM tvisit WHERE KodePT=? AND KodeSales=? AND DATE(TglEntry) = ?
		`
	row, err := rp.DB.QueryContext(ctx, qry, sKodePT, sKodeSales, time.Now().Format("2006-01-02"))
	if err != nil {
		log.Println(err)
		return nil, 0, err
	}
	var VisitData []models.SalesVisit
	for row.Next() {
		var visit models.SalesVisit
		err = row.Scan(
			&visit.UID,
			&visit.KodePT,
			&visit.KodeSales,
			&visit.KodeCust,
			&visit.NamaCust,
			&visit.TglEntry,
			&visit.CheckIn,
			&visit.CheckOut,
			&visit.CheckInLocation.Lat,
			&visit.CheckInLocation.Lng,
			&visit.LamaKunjungan,
			&visit.CheckOutLocation.Lat,
			&visit.CheckOutLocation.Lng,
			&visit.CheckInOut,
			&visit.CheckInDalamArea,
			&visit.CheckOutDalamArea,
		)
		if err != nil {
			log.Println(err)
			return nil, 0, err
		}
		VisitData = append(VisitData, visit)
	}
	return &VisitData, 0, err
}
func (rp *portalDBRepo) SalesCheckIn(sKodePT string, CheckInData models.SalesCheckIn) (*models.SalesVisit, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()

	qry := `INSERT INTO tvisit(
                   UID,
                   KodePT,
                   KodeSales,
                   KodeCust,
                   NamaCust,
                   TglEntry,
                   CheckIn,
                   CheckInLat,
                   CheckInLng,                   
                   CheckInDalamArea

			) VALUES(?,?,?,?,?,NOW(),?,?,?,?)`
	_, err := rp.DB.ExecContext(ctx, qry,
		CheckInData.UID,
		sKodePT,
		CheckInData.KodeSales,
		CheckInData.KodeCust,
		CheckInData.NamaCust,
		CheckInData.CheckIn,
		CheckInData.CheckInLoc.Lat,
		CheckInData.CheckInLoc.Lng,
		CheckInData.CheckInDalamArea)
	if err != nil {
		log.Println(err)
		return nil, err
	}
	qry = `SELECT 
				UID,
				KodeCust,
				NamaCust,
				TglEntry,
				CheckIn,
				CheckInLat,
				CheckInLng,
				CheckInDalamArea
			FROM tvisit WHERE UID=?
				`
	var VisitData models.SalesVisit
	err = rp.DB.QueryRowContext(ctx, qry, CheckInData.UID).Scan(
		&VisitData.UID,
		&VisitData.KodeCust,
		&VisitData.NamaCust,
		&VisitData.TglEntry,
		&VisitData.CheckIn,
		&VisitData.CheckInLocation.Lat,
		&VisitData.CheckInLocation.Lng,
		&VisitData.CheckInDalamArea,
	)
	if err != nil {
		log.Println(err)
		return nil, err
	}

	return &VisitData, nil
}

func (rp *portalDBRepo) SalesGetProduct(sKodePT string) (*[]models.Product, int, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()
	qry := `SELECT 	
				KodeBrg,
				NamaBrg,
				StockAkhir,
				Satuan,
				Konv
			FROM tproduk WHERE KodePT=? 
`
	row, err := rp.DB.QueryContext(ctx, qry, sKodePT)
	if err != nil {
		log.Println(err)
		return nil, 0, err
	}
	var ProductList []models.Product
	for row.Next() {
		var ProductRow models.Product
		err = row.Scan(
			&ProductRow.KodeBrg,
			&ProductRow.NamaBrg,
			&ProductRow.Stock,
			&ProductRow.Satuan,
			&ProductRow.Konv,
		)
		if err != nil {
			log.Println(err)
			return nil, 0, err
		}
		ProductList = append(ProductList, ProductRow)
	}
	return &ProductList, len(ProductList), nil
}
func (rp *portalDBRepo) SalesCheckOut(sKodePT string, CheckOutData models.SalesCheckOut) (*models.SalesVisit, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()
	qry := `UPDATE tvisit SET 
				CheckOut =?,
				CheckOutLat =?,
				CheckOutLng = ?,	
				CheckOutDalamArea=?
				WHERE UID = ?
				`
	_, err := rp.DB.ExecContext(ctx, qry,
		CheckOutData.CheckOut,
		CheckOutData.CheckOutLoc.Lat,
		CheckOutData.CheckOutLoc.Lng,
		CheckOutData.CheckOutDalamArea,
		CheckOutData.UID,
	)
	if err != nil {
		log.Println(err)
		return nil, err
	}
	qry = `SELECT 
				UID,		
				KodeSales,
				KodeCust,
				NamaCust,
				TglEntry,
				CheckIn,
				CheckOut,
				CheckInLat,
				CheckInLng,
				IFNULL(Lama,""),
				CheckOutLat,
				CheckOutLng,
				CheckInOut,
				CheckInDalamArea,
				CheckOutDalamArea

			FROM tvisit WHERE UID=?
				`
	var VisitData models.SalesVisit
	err = rp.DB.QueryRowContext(ctx, qry, CheckOutData.UID).Scan(
		&VisitData.UID,
		&VisitData.KodeSales,
		&VisitData.KodeCust,
		&VisitData.NamaCust,
		&VisitData.TglEntry,
		&VisitData.CheckIn,
		&VisitData.CheckOut,
		&VisitData.CheckInLocation.Lat,
		&VisitData.CheckInLocation.Lng,
		&VisitData.LamaKunjungan,
		&VisitData.CheckOutLocation.Lat,
		&VisitData.CheckOutLocation.Lng,
		&VisitData.CheckInOut,
		&VisitData.CheckInDalamArea,
		&VisitData.CheckOutDalamArea,
	)
	if err != nil {
		log.Println(err)
		return nil, err
	}
	return &VisitData, nil
}

func (rp *portalDBRepo) ChangePassword(sKodePT string, newcredential *models.ChangeCredential) (models.ChangePasswordResponse, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()
	qry := `SELECT dbSalesPasswordUpdate(?,?, ?,?)`
	row, err := rp.DB.QueryContext(ctx, qry, sKodePT, newcredential.Username, newcredential.OldPassword, newcredential.NewPassword)
	if err != nil {
		log.Println(err)
		return models.ChangePasswordResponse{PasswordChanged: false, LoginState: models.LoggedIn.String()}, err
	}
	var Result string
	for row.Next() {
		err = row.Scan(&Result)
		if err != nil {
			log.Println(err)
			return models.ChangePasswordResponse{PasswordChanged: false, LoginState: models.LoggedIn.String()}, err
		}
	}
	if Result == "Password Changed" {
		return models.ChangePasswordResponse{PasswordChanged: true, LoginState: models.LoggedIn.String()}, nil
	} else {
		return models.ChangePasswordResponse{PasswordChanged: false, LoginState: models.LoggedIn.String()}, errors.New("unauthorized")
	}
}
func (rp *portalDBRepo) GetSalesByNoHP(sNoHP string) (*models.SalesData, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()
	qry := ` SELECT IFNULL(KodePT,"") KodePT,IFNULL(KodeSales,"") KodeSales, IFNULL(NamaSales,"") NamaSales, IFNULL(NoHP,"") NoHandphone, IFNULL(Area,"") Area 
			FROM tsales WHERE formatPhoneNumber(NoHP) = ? AND Aktif="Ya"`
	var Sales models.SalesData
	err := rp.DB.QueryRowContext(ctx, qry, sNoHP).Scan(&Sales.KodePT, &Sales.KodeSales, &Sales.NamaSales, &Sales.NoHandphone, &Sales.Area)
	if err != nil {
		log.Println(err)
		return nil, err
	}
	return &Sales, nil
}

func (rp *portalDBRepo) RequestResetPassword(sNoHP string) (string, string, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()
	NewPW := generateRandomString(6)
	qry := `UPDATE tsales SET Password=SHA1('` + NewPW + `') WHERE formatPhoneNumber(NoHP)=? AND Aktif="Ya"`
	log.Println(sNoHP)
	log.Println(qry)
	_, err := rp.DB.ExecContext(ctx, qry, sNoHP)
	if err != nil {
		log.Println(err)
		return "", sNoHP, err
	}
	Sales, err := rp.GetSalesByNoHP(sNoHP)
	if err != nil {
		log.Println(err)
		return "", sNoHP, err
	}
	return NewPW, Sales.KodeSales, nil
	//qry := `CALL dbSalesPasswordReset(?,?,?)`
	//row, err := rp.DB.QueryContext(ctx, qry, sKodePT, sKodeSales, "Request reset password")
	//if err != nil {
	//	log.Println(err)
	//	return models.ResetPasswordResponse{RequestSent: false, RequestID: "", RequestStatus: models.RPError.String()}, err
	//}
	//var Result models.ResetPasswordResponse
	//for row.Next() {
	//	err = row.Scan(&Result.RequestSent, &Result.RequestStatus, &Result.RequestID)
	//	if err != nil {
	//		log.Println(err)
	//		return models.ResetPasswordResponse{RequestSent: false, RequestID: "", RequestStatus: models.RPError.String()}, err
	//	}
	//}
	//
	//return Result, nil

}
func (rp *portalDBRepo) ApproveResetPassword(sKodePT string, sNoRequest string) (models.ResetPasswordResponse, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()
	qry := `CALL dbSalesPasswordApproveReset(?,?)`
	row, err := rp.DB.QueryContext(ctx, qry, sKodePT, sNoRequest)
	if err != nil {
		log.Println(err)
		return models.ResetPasswordResponse{RequestSent: false, RequestID: "", RequestStatus: models.RPError.String()}, err
	}
	var Result models.ResetPasswordResponse
	for row.Next() {
		err = row.Scan(&Result.RequestSent, &Result.RequestStatus, &Result.RequestID)
		if err != nil {
			log.Println(err)
			return models.ResetPasswordResponse{RequestSent: false, RequestID: "", RequestStatus: models.RPError.String()}, err
		}
	}

	return Result, nil

}
func generateRandomString(length int) string {
	const charset = "abcdefghkmnpqrstuvwxyz23456789"
	seededRand := rand.New(rand.NewSource(time.Now().UnixNano()))

	b := make([]byte, length)
	for i := range b {
		b[i] = charset[seededRand.Intn(len(charset))]
	}
	return string(b)
}
func (rp *portalDBRepo) NeedSync(sKodePT string, sKodeSales string, sTransID string, nCounterToCheck int) (bool, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()

	qry := `SELECT dbNeedSync(?,?,?,?)`
	row, err := rp.DB.QueryContext(ctx, qry, sKodePT, sKodeSales, sTransID, nCounterToCheck)
	if err != nil {
		log.Println(err)
		return true, err
	}
	var sBool string

	for row.Next() {
		err = row.Scan(&sBool)
		if err != nil {
			log.Println(err)
			return true, err
		}
	}
	return sBool == "TRUE", nil
}
func (rp *portalDBRepo) UpdateSyncID(sKodePT string, sKodeSales string, sTransID string, NewID int) error {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()
	qry := `CALL dbSyncSetID(?,?,?,?)`

	_, err := rp.DB.ExecContext(ctx, qry, sKodePT, sKodeSales, sTransID, NewID)
	if err != nil {
		log.Println(err)
		return err
	}
	return nil
}
func (rp *portalDBRepo) CustomerListGet(sKodePT string) (*[]models.CustomerData, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()
	qry := `CALL dbCustomerListGet(?)`
	row, err := rp.DB.QueryContext(ctx, qry, sKodePT)
	if err != nil {
		log.Println(err)
		return nil, err
	}

	var CustomerData []models.CustomerData

	defer func() {
		err := row.Close()
		if err != nil {
			log.Println("Error closing rows!")
		}
	}()
	for row.Next() {
		var Customer models.CustomerData
		err = row.Scan(
			&Customer.UID,
			&Customer.KodePT,
			&Customer.KodeCust,
			&Customer.NamaCust,
			&Customer.AlamatCust,
			&Customer.GPSLocation.Lat,
			&Customer.GPSLocation.Lng,
			&Customer.Kota,
			&Customer.Telp,
			&Customer.Kecamatan,
			&Customer.Pemilik,
			&Customer.NoKTP,
			&Customer.FileFotoKTP,
			&Customer.FileFotoToko1,
			&Customer.FileFotoToko2,
			&Customer.AlamatPemilik,
			&Customer.TipeCust,
			&Customer.CreditLimit,
			&Customer.Piutang,
			&Customer.JmlArmada,
			&Customer.LevelToko,
			&Customer.SistemPembayaran,
			&Customer.StatusKepemilikan,
			&Customer.KodeSales,
			&Customer.NPWP,
			&Customer.Aktif,
			&Customer.FlagPiutang45,
			&Customer.FlagLebihPlafon,
		)
		if err != nil {
			log.Println(err)
			return nil, err
		}
		CustomerData = append(CustomerData, Customer)
	}
	return &CustomerData, nil
}
func (rp *portalDBRepo) CustomerDelete(sKodePT string, sUID string) error {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()
	//TODO Implement dbDeleteCustomerByUID

	qry := `CALL DeleteCustomerByUID(?,?)`
	_, err := rp.DB.ExecContext(ctx, qry, sKodePT, sUID)
	if err != nil {
		return err
	}
	return nil
}
func (rp *portalDBRepo) CustomerApprove(sUID string, ApprovalStatus models.ApprovalStatus) (error, *models.ApprovalStatus) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()

	qry := `CALL dbCustomerApprove(?)`
	var ApprovalResult models.ApprovalStatus
	err := rp.DB.QueryRowContext(ctx, qry, sUID).Scan(&ApprovalResult)
	if err != nil {
		return err, nil
	}
	return nil, &ApprovalResult
}
func (rp *portalDBRepo) CustomerGetByKodeCust(sKodePT string, sKodeCust string) (*models.CustomerData, int, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()
	qry := `CALL dbCustomerGetByKodeCust(?,?)`
	row, err := rp.DB.QueryContext(ctx, qry, sKodePT, sKodeCust)
	if err != nil {
		log.Println(err)
		return nil, 0, err
	}

	var Customer models.CustomerData

	defer func() {
		err := row.Close()
		if err != nil {
			log.Println("Error closing rows!")
		}
	}()
	nCount := 0
	for row.Next() {
		nCount++
		err = row.Scan(
			&Customer.UID,
			&Customer.KodePT,
			&Customer.KodeCust,
			&Customer.NamaCust,
			&Customer.AlamatCust,
			&Customer.GPSLocation.Lat,
			&Customer.GPSLocation.Lng,
			&Customer.Kota,
			&Customer.Telp,
			&Customer.Kecamatan,
			&Customer.Pemilik,
			&Customer.NoKTP,
			&Customer.FileFotoKTP,
			&Customer.FileFotoToko1,
			&Customer.FileFotoToko2,
			&Customer.AlamatPemilik,
			&Customer.TipeCust,
			&Customer.CreditLimit,
			&Customer.Piutang,
			&Customer.JmlArmada,
			&Customer.LevelToko,
			&Customer.SistemPembayaran,
			&Customer.StatusKepemilikan,
			&Customer.KodeSales,
			&Customer.NPWP,
			&Customer.Aktif,
			&Customer.FlagPiutang45,
			&Customer.FlagLebihPlafon,
		)
		if err != nil {
			log.Println(err)
			return nil, 0, err
		}

	}
	return &Customer, nCount, nil
}

func (rp *portalDBRepo) RKSListGet(sKodePT string) (*[]models.RKSData, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()
	qry := `CALL dbRKSListGet(?)`
	row, err := rp.DB.QueryContext(ctx, qry, sKodePT)
	if err != nil {
		log.Println(err)
		return nil, err
	}

	var RKSData []models.RKSData

	defer func() {
		err := row.Close()
		if err != nil {
			log.Println("Error closing rows!")
		}
	}()
	for row.Next() {
		var RKS models.RKSData
		err = row.Scan(&RKS.KodePT, &RKS.KodeSales, &RKS.Hari, &RKS.KodeCust, &RKS.NamaCust, &RKS.CreditLimit, &RKS.SisaCredit, &RKS.TotalPiutang, &RKS.PiutangLebih45)
		if err != nil {
			log.Println(err)
			return nil, err
		}
		RKSData = append(RKSData, RKS)
	}
	return &RKSData, nil
}
func (rp *portalDBRepo) RKSSetData(sKodePT string, RKSData []models.RKSData) (int, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()
	log.Println("CALL dbRKSEmpty()")
	qry := `CALL dbRKSEmpty(?)`

	_, err := rp.DB.ExecContext(ctx, qry, sKodePT)
	if err != nil {
		log.Println(err)
		return 0, err
	}
	log.Println("Adding RKS from server")
	for _, RKS := range RKSData {

		qry = `CALL dbRKSAdd(?,?,?,?,?,?,?,?,?)`
		_, err := rp.DB.ExecContext(ctx, qry, RKS.KodePT, RKS.KodeSales, RKS.Hari, RKS.KodeCust, RKS.NamaCust, RKS.CreditLimit, RKS.SisaCredit, RKS.TotalPiutang, RKS.PiutangLebih45)
		if err != nil {
			log.Println(err)
			return 0, err
		}
	}
	log.Println("Getting RKS from portal")
	NewRKS, err := rp.RKSListGet(sKodePT)
	if err != nil {
		log.Println(err)
		return 0, err
	}

	return len(*NewRKS), nil
}
func (rp *portalDBRepo) RKSListGetBySales(sKodePT string, sKodeSales string) (*[]models.RKSData, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()
	qry := `CALL dbRKSListGetBySales(?,NOW(),? )`
	row, err := rp.DB.QueryContext(ctx, qry, sKodePT, sKodeSales)
	if err != nil {
		log.Println(err)
		return nil, err
	}

	var RKSData []models.RKSData

	defer func() {
		err := row.Close()
		if err != nil {
			log.Println("Error closing rows!")
		}
	}()
	for row.Next() {
		var RKS models.RKSData
		err = row.Scan(&RKS.KodePT, &RKS.KodeSales, &RKS.Hari, &RKS.KodeCust, &RKS.NamaCust, &RKS.CreditLimit, &RKS.SisaCredit, &RKS.TotalPiutang, &RKS.PiutangLebih45)
		if err != nil {
			log.Println(err)
			return nil, err
		}
		Customer, recCount, err := rp.CustomerGetByKodeCust(RKS.KodePT, RKS.KodeCust)
		if err != nil {
			log.Print(err)
		} else {
			if recCount == 1 {
				RKS.CustomerData = *Customer
			}
		}
		RKSData = append(RKSData, RKS)
	}

	return &RKSData, nil
}
func (rp *portalDBRepo) CustomerListGetBySales(sKodePT string, sKodeSales string) (*[]models.CustomerData, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()
	qry := `CALL dbCustomerListGetBySales(?,? )`
	row, err := rp.DB.QueryContext(ctx, qry, sKodePT, sKodeSales)
	if err != nil {
		log.Println(err)
		return nil, err
	}

	var CustomerData []models.CustomerData

	defer func() {
		err := row.Close()
		if err != nil {
			log.Println("Error closing rows!")
		}
	}()

	for row.Next() {
		var Customer models.CustomerData
		err = row.Scan(
			&Customer.UID,
			&Customer.KodePT,
			&Customer.KodeCust,
			&Customer.NamaCust,
			&Customer.AlamatCust,
			&Customer.GPSLocation.Lat,
			&Customer.GPSLocation.Lng,
			&Customer.Kota,
			&Customer.Telp,
			&Customer.Kecamatan,
			&Customer.Pemilik,
			&Customer.NoKTP,
			&Customer.FileFotoKTP,
			&Customer.FileFotoToko1,
			&Customer.FileFotoToko2,
			&Customer.AlamatPemilik,
			&Customer.TipeCust,
			&Customer.CreditLimit,
			&Customer.Piutang,
			&Customer.JmlArmada,
			&Customer.LevelToko,
			&Customer.SistemPembayaran,
			&Customer.StatusKepemilikan,
			&Customer.KodeSales,
			&Customer.NPWP,
			&Customer.Aktif,
			&Customer.FlagPiutang45,
			&Customer.FlagLebihPlafon)
		if err != nil {
			log.Println(err)
			return nil, err
		}
		//Customer.KodePT = sKodePT
		CustomerData = append(CustomerData, Customer)
	}

	return &CustomerData, nil
}

func (rp *portalDBRepo) CustomerGetBySalesKodeCust(sKodePT string, sKodeSales string, sKodeCust string) (*models.CustomerData, int, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()
	qry := `CALL dbCustomerGetBySalesKodeCust(?,?, ?)`
	row, err := rp.DB.QueryContext(ctx, qry, sKodePT, sKodeSales, sKodeCust)
	if err != nil {
		log.Println(err)
		return nil, 0, err
	}

	var Customer models.CustomerData

	defer func() {
		err := row.Close()
		if err != nil {
			log.Println("Error closing rows!")
		}
	}()
	nCount := 0
	for row.Next() {
		nCount++
		err = row.Scan(
			&Customer.UID,
			&Customer.KodePT,
			&Customer.KodeCust,
			&Customer.NamaCust,
			&Customer.AlamatCust,
			&Customer.GPSLocation.Lat,
			&Customer.GPSLocation.Lng,
			&Customer.Kota,
			&Customer.Telp,
			&Customer.Kecamatan,
			&Customer.Pemilik,
			&Customer.NoKTP,
			&Customer.FileFotoKTP,
			&Customer.FileFotoToko1,
			&Customer.FileFotoToko2,
			&Customer.FileFotoToko3,
			&Customer.AlamatPemilik,
			&Customer.TipeCust,
			&Customer.CreditLimit,
			&Customer.Piutang,
			&Customer.JmlArmada,
			&Customer.LevelToko,
			&Customer.SistemPembayaran,
			&Customer.StatusKepemilikan,
			&Customer.KodeSales,
			&Customer.NPWP,
			&Customer.Aktif,
			&Customer.FlagPiutang45,
			&Customer.FlagLebihPlafon,
		)
		if err != nil {
			log.Println(err)
			return nil, 0, err
		}

	}
	return &Customer, nCount, nil
}
func (rp *portalDBRepo) CustomerGetByUID(sUID string) (*models.CustomerData, int, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()

	var Customer models.CustomerData
	qry := `CALL dbCustomerGetByUID(?)`
	err := rp.DB.QueryRowContext(ctx, qry, sUID).Scan(
		&Customer.UID,
		&Customer.KodePT,
		&Customer.KodeCust,
		&Customer.NamaCust,
		&Customer.AlamatCust,
		&Customer.GPSLocation.Lat,
		&Customer.GPSLocation.Lng,
		&Customer.Kota,
		&Customer.Telp,
		&Customer.Kecamatan,
		&Customer.Pemilik,
		&Customer.NoKTP,
		&Customer.FileFotoKTP,
		&Customer.FileFotoToko1,
		&Customer.FileFotoToko2,
		&Customer.AlamatPemilik,
		&Customer.TipeCust,
		&Customer.CreditLimit,
		&Customer.Piutang,
		&Customer.JmlArmada,
		&Customer.LevelToko,
		&Customer.SistemPembayaran,
		&Customer.StatusKepemilikan,
		&Customer.KodeSales,
		&Customer.NPWP,
		&Customer.Aktif,
		&Customer.FlagPiutang45,
		&Customer.FlagLebihPlafon,
	)
	if err != nil {
		return nil, 0, err
	}
	return &Customer, 1, nil
}
func (rp *portalDBRepo) CustomerSet(sKodePT string, CustomerData []models.CustomerData) (int, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()

	for _, Customer := range CustomerData {

		qry := `CALL dbCustomerSet(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)`
		_, err := rp.DB.ExecContext(ctx, qry,
			Customer.KodePT,
			Customer.KodeCust,
			Customer.NamaCust,
			Customer.AlamatCust,
			Customer.GPSLocation.Lat,
			Customer.GPSLocation.Lng,
			Customer.Kota,
			Customer.Telp,
			Customer.Kecamatan,
			Customer.Pemilik,
			Customer.NoKTP,
			Customer.FileFotoKTP,
			Customer.FileFotoToko1,
			Customer.FileFotoToko2,
			Customer.AlamatPemilik,
			Customer.TipeCust,
			Customer.CreditLimit,
			Customer.Piutang,
			Customer.JmlArmada,
			Customer.LevelToko,
			Customer.SistemPembayaran,
			Customer.StatusKepemilikan,
			Customer.KodeSales,
			Customer.NPWP,
			Customer.Aktif,
			Customer.FlagPiutang45,
			Customer.FlagLebihPlafon,
		)
		if err != nil {
			log.Println(err)
			return 0, err
		}
	}
	NewCustomerList, err := rp.CustomerListGet(sKodePT)
	if err != nil {
		return 0, err
	}

	return len(*NewCustomerList), nil
}
func (rp *portalDBRepo) CustomerUpdate(sKodePT string, Customer models.CustomerData) (*models.CustomerData, int, error) {

	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()
	path := filepath.Join(".", "UploadedPhotos")

	if _, err := os.Stat(path); os.IsNotExist(err) {
		err = os.Mkdir(path, os.ModePerm)
		if err != nil {
			log.Print(err, "\r\n")
		}
	}
	log.Println("Persiapan Upload Foto")
	var bFotoKTP []byte
	var bFotoToko1 []byte
	var bFotoToko2 []byte
	var bFotoToko3 []byte
	var nFotoKTP string
	var nFotoToko1 string
	var nFotoToko2 string
	var nFotoToko3 string
	var fileFotoKTP *os.File
	var fileFotoToko1 *os.File
	var fileFotoToko2 *os.File
	var fileFotoToko3 *os.File
	if len(Customer.FileFotoKTP) > 100 {
		bFotoKTP, _ = base64.StdEncoding.DecodeString(Customer.FileFotoKTP)
		nFotoKTP = fmt.Sprintf("%x", md5.Sum([]byte(uuid.New().String()))) + ".jpg"
		fileFotoKTP, _ = os.Create(path + "\\" + nFotoKTP)
		defer fileFotoKTP.Close()
		_, _ = fileFotoKTP.Write(bFotoKTP)
	} else {
		nFotoKTP = Customer.FileFotoKTP
	}
	if len(Customer.FileFotoToko1) > 100 {
		bFotoToko1, _ = base64.StdEncoding.DecodeString(Customer.FileFotoToko1)
		nFotoToko1 = fmt.Sprintf("%x", md5.Sum([]byte(uuid.New().String()))) + ".jpg"
		fileFotoToko1, _ = os.Create(path + "\\" + nFotoToko1)
		defer fileFotoToko1.Close()
		_, _ = fileFotoToko1.Write(bFotoToko1)
	} else {
		nFotoToko1 = Customer.FileFotoToko1
	}
	if len(Customer.FileFotoToko2) > 100 {
		bFotoToko2, _ = base64.StdEncoding.DecodeString(Customer.FileFotoToko2)
		nFotoToko2 = fmt.Sprintf("%x", md5.Sum([]byte(uuid.New().String()))) + ".jpg"
		fileFotoToko2, _ = os.Create(path + "\\" + nFotoToko2)
		defer fileFotoToko2.Close()
		_, _ = fileFotoToko2.Write(bFotoToko2)
	} else {
		nFotoToko2 = Customer.FileFotoToko2
	}
	if len(Customer.FileFotoToko3) > 100 {
		bFotoToko3, _ = base64.StdEncoding.DecodeString(Customer.FileFotoToko3)
		nFotoToko3 = fmt.Sprintf("%x", md5.Sum([]byte(uuid.New().String()))) + ".jpg"
		fileFotoToko3, _ = os.Create(path + "\\" + nFotoToko3)
		defer fileFotoToko3.Close()
		_, _ = fileFotoToko3.Write(bFotoToko3)
	} else {
		nFotoToko3 = Customer.FileFotoToko3
	}

	Customer.FileFotoKTP = nFotoKTP
	Customer.FileFotoToko1 = nFotoToko1
	Customer.FileFotoToko2 = nFotoToko2
	Customer.FileFotoToko3 = nFotoToko3
	Customer.Aktif = "Ya"
	log.Println("Upload Foto Selesai")
	qry := `CALL dbCustomerAddUpdate(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)`
	_, err := rp.DB.ExecContext(ctx, qry,

		Customer.UID,
		sKodePT,
		Customer.KodeCust,
		Customer.NamaCust,
		Customer.AlamatCust,
		Customer.GPSLocation.Lat,
		Customer.GPSLocation.Lng,
		Customer.Kota,
		Customer.Telp,
		Customer.Kecamatan,
		Customer.Pemilik,
		Customer.NoKTP,
		Customer.FileFotoKTP,
		Customer.FileFotoToko1,
		Customer.FileFotoToko2,
		Customer.FileFotoToko3,
		Customer.AlamatPemilik,
		Customer.TipeCust,
		Customer.CreditLimit,
		Customer.Piutang,
		Customer.JmlArmada,
		Customer.LevelToko,
		Customer.SistemPembayaran,
		Customer.StatusKepemilikan,
		Customer.KodeSales,
		Customer.NPWP,
		Customer.Aktif,
	)
	if err != nil {
		log.Println(err)
		return nil, 0, err
	}

	NewCustomer, recCount, err := rp.CustomerGetBySalesKodeCust(sKodePT, Customer.KodeSales, Customer.KodeCust)
	if err != nil {
		log.Println(err)
		return nil, 0, err
	} else if recCount == 0 {
		log.Println("No record")
		return nil, 0, errors.New("No record found")
	}

	return NewCustomer, 1, nil
}

const charset = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"

func (rp *portalDBRepo) CustomerAdd(sKodePT string, Customer models.CustomerData) (*models.CustomerData, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()
	path := filepath.Join(".", "UploadedPhotos")

	if _, err := os.Stat(path); os.IsNotExist(err) {
		err = os.Mkdir(path, os.ModePerm)
		if err != nil {
			log.Print(err, "\r\n")
		}
	}
	// Decode the string
	bFotoKTP, _ := base64.StdEncoding.DecodeString(Customer.FileFotoKTP)
	bFotoToko1, _ := base64.StdEncoding.DecodeString(Customer.FileFotoToko1)
	bFotoToko2, _ := base64.StdEncoding.DecodeString(Customer.FileFotoToko2)
	bFotoToko3, _ := base64.StdEncoding.DecodeString(Customer.FileFotoToko3)

	nFotoKTP := fmt.Sprintf("%x", md5.Sum([]byte(uuid.New().String()))) + ".jpg"
	nFotoToko1 := fmt.Sprintf("%x", md5.Sum([]byte(uuid.New().String()))) + ".jpg"
	nFotoToko2 := fmt.Sprintf("%x", md5.Sum([]byte(uuid.New().String()))) + ".jpg"
	nFotoToko3 := fmt.Sprintf("%x", md5.Sum([]byte(uuid.New().String()))) + ".jpg"

	fileFotoKTP, _ := os.Create(path + "\\" + nFotoKTP)
	fileFotoToko1, _ := os.Create(path + "\\" + nFotoToko1)
	fileFotoToko2, _ := os.Create(path + "\\" + nFotoToko2)
	fileFotoToko3, _ := os.Create(path + "\\" + nFotoToko3)
	defer fileFotoKTP.Close()
	defer fileFotoToko1.Close()
	defer fileFotoToko2.Close()
	defer fileFotoToko3.Close()
	_, _ = fileFotoKTP.Write(bFotoKTP)
	_, _ = fileFotoToko1.Write(bFotoToko1)
	_, _ = fileFotoToko2.Write(bFotoToko2)
	_, _ = fileFotoToko3.Write(bFotoToko3)
	Customer.FileFotoKTP = nFotoKTP
	Customer.FileFotoToko1 = nFotoToko1
	Customer.FileFotoToko2 = nFotoToko2
	Customer.FileFotoToko3 = nFotoToko3
	Customer.Aktif = "Ya"
	qry := `CALL dbCustomerAdd(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)`
	_, err := rp.DB.ExecContext(ctx, qry,

		Customer.UID,
		sKodePT,
		Customer.KodeCust,
		Customer.NamaCust,
		Customer.AlamatCust,
		Customer.GPSLocation.Lat,
		Customer.GPSLocation.Lng,
		Customer.Kota,
		Customer.Telp,
		Customer.Kecamatan,
		Customer.Pemilik,
		Customer.NoKTP,
		Customer.FileFotoKTP,
		Customer.FileFotoToko1,
		Customer.FileFotoToko2,
		Customer.FileFotoToko3,
		Customer.AlamatPemilik,
		Customer.TipeCust,
		Customer.CreditLimit,
		Customer.Piutang,
		Customer.JmlArmada,
		Customer.LevelToko,
		Customer.SistemPembayaran,
		Customer.StatusKepemilikan,
		Customer.KodeSales,
		Customer.NPWP,
		Customer.Aktif,
	)
	if err != nil {
		log.Println(err)
		return nil, err
	}

	NewCustomer, _, err := rp.CustomerGetBySalesKodeCust(sKodePT, Customer.KodeSales, Customer.KodeCust)
	if err != nil {
		return nil, err
	}
	return NewCustomer, nil
}
func (rp *portalDBRepo) SalesOrderListGetBySales(sKodePT string, sKodeSales string) (*[]models.OrderData, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()
	qry := `CALL dbSalesOrderListGetBySales(?,? )`

	row, err := rp.DB.QueryContext(ctx, qry, sKodePT, sKodeSales)
	if err != nil {
		log.Println(err)
		return nil, err
	}

	var OrderData []models.OrderData
	var OrderItems []models.OrderItems

	defer func() {
		err := row.Close()
		if err != nil {
			log.Println("Error closing rows!")
		}
	}()
	var PrevOrderUID string

	for row.Next() {
		var SalesOrder models.OrderData
		var SalesDetail models.OrderItems
		err = row.Scan(
			&SalesOrder.UID,
			&SalesOrder.KodePT,
			&SalesOrder.KodeSales,
			&SalesOrder.NoOrder,
			&SalesOrder.NoSO,
			&SalesOrder.KodeCust,
			&SalesOrder.NamaCust,
			&SalesOrder.TglSO,
			&SalesOrder.TglJT,
			&SalesOrder.Nilai,
			&SalesOrder.Keterangan,
			&SalesOrder.ApprovalStatus,
			&SalesDetail.TipeBrg,
			&SalesDetail.NoUrut,
			&SalesDetail.KodeBrg,
			&SalesDetail.NamaBrg,
			&SalesDetail.Satuan,
			&SalesDetail.Jml,
			&SalesDetail.Konv,
			&SalesDetail.Harga,
			&SalesDetail.DiskonPersen,
			&SalesDetail.DiskonRupiah,
			&SalesDetail.TipeDiskon,
			&SalesDetail.HargaDiskon,
			&SalesDetail.Total)
		if err != nil {
			log.Println(err)
			return nil, err
		}
		if PrevOrderUID != SalesOrder.UID {
			OrderItems = nil
		}
		OrderItems = append(OrderItems, SalesDetail)

		if PrevOrderUID != SalesOrder.UID {
			SalesOrder.Items = OrderItems
			OrderData = append(OrderData, SalesOrder)
		} else {
			SalesOrder.Items = OrderItems
			OrderData[len(OrderData)-1] = SalesOrder
		}
		PrevOrderUID = SalesOrder.UID
	}

	return &OrderData, nil
}

func (rp *portalDBRepo) SalesOrderGetByUID(sUID string) (*models.OrderData, int, error) {
	ctx, cancel := context.WithTimeout(context.Background(), 5*rp.App.ContextTime)
	defer cancel()

	qry := `CALL dbSalesOrderGetByUID(?)`
	row, err := rp.DB.QueryContext(ctx, qry, sUID)

	var OrderItems []models.OrderItems

	defer func() {
		err := row.Close()
		if err != nil {
			log.Println("Error closing rows!")
		}
	}()
	var SalesOrder models.OrderData
	for row.Next() {
		var SalesDetail models.OrderItems
		err = row.Scan(
			&SalesOrder.UID,
			&SalesOrder.KodePT,
			&SalesOrder.KodeSales,
			&SalesOrder.NoOrder,
			&SalesOrder.NoSO,
			&SalesOrder.KodeCust,
			&SalesOrder.NamaCust,
			&SalesOrder.TglSO,
			&SalesOrder.TglJT,
			&SalesOrder.Nilai,
			&SalesOrder.Keterangan,
			&SalesOrder.ApprovalStatus,
			&SalesDetail.TipeBrg,
			&SalesDetail.NoUrut,
			&SalesDetail.KodeBrg,
			&SalesDetail.NamaBrg,
			&SalesDetail.Satuan,
			&SalesDetail.Jml,
			&SalesDetail.Konv,
			&SalesDetail.Harga,
			&SalesDetail.DiskonPersen,
			&SalesDetail.DiskonRupiah,
			&SalesDetail.TipeDiskon,
			&SalesDetail.HargaDiskon,
			&SalesDetail.Total)
		if err != nil {
			log.Println(err)
			return nil, 0, err
		}

		OrderItems = append(OrderItems, SalesDetail)
		SalesOrder.Items = OrderItems
	}
	return &SalesOrder, 1, nil
}
func (rp *portalDBRepo) SalesOrderGetByNoOrder(sKodePT string, sNoOrder string, sKodeSales string) (*models.OrderData, int, error) {
	ctx, cancel := context.WithTimeout(context.Background(), 5*rp.App.ContextTime)
	defer cancel()

	qry := `CALL dbSalesOrderGetByNoOrder(?,?,?)`
	row, err := rp.DB.QueryContext(ctx, qry, sKodePT, sNoOrder, sKodeSales)
	if err != nil {
		return nil, 0, err
	}
	var OrderItems []models.OrderItems

	defer func() {
		err := row.Close()
		if err != nil {
			log.Println("Error closing rows!")
		}
	}()
	var SalesOrder models.OrderData
	for row.Next() {
		var SalesDetail models.OrderItems
		err = row.Scan(
			&SalesOrder.UID,
			&SalesOrder.KodePT,
			&SalesOrder.KodeSales,
			&SalesOrder.NoOrder,
			&SalesOrder.NoSO,
			&SalesOrder.KodeCust,
			&SalesOrder.TglSO,
			&SalesOrder.TglJT,
			&SalesOrder.Nilai,
			&SalesOrder.Keterangan,
			&SalesOrder.ApprovalStatus,
			&SalesDetail.TipeBrg,
			&SalesDetail.NoUrut,
			&SalesDetail.KodeBrg,
			&SalesDetail.NamaBrg,
			&SalesDetail.Satuan,
			&SalesDetail.Jml,
			&SalesDetail.Konv,
			&SalesDetail.Harga,
			&SalesDetail.DiskonPersen,
			&SalesDetail.DiskonRupiah,
			&SalesDetail.TipeDiskon,
			&SalesDetail.HargaDiskon,
			&SalesDetail.Total)
		if err != nil {
			log.Println(err)
			return nil, 0, err
		}

		OrderItems = append(OrderItems, SalesDetail)
		SalesOrder.Items = OrderItems
	}
	return &SalesOrder, 1, nil
}
func (rp *portalDBRepo) SalesOrderAdd(sKodePT string, sKodeSales string, SalesOrder models.OrderData) (*models.OrderData, int, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()

	if SalesOrder.UID == "" {
		sUID, sNomor, err := rp.NoBaruWithUID("SO", time.Now())
		if err != nil {
			return nil, 0, err
		}
		SalesOrder.UID = sUID
		SalesOrder.KodePT = sKodePT
		SalesOrder.KodeSales = sKodeSales
		SalesOrder.NoOrder = sNomor
	}
	qry := `DELETE FROM tr_so_d WHERE UID=?`
	_, err := rp.DB.ExecContext(ctx, qry, SalesOrder.UID)
	if err != nil {
		return nil, 0, err
	}
	for _, OrderItems := range SalesOrder.Items {
		if OrderItems.KodeBrg == "" {
			OrderItems.TipeBrg = models.NewRequest
		} else {
			OrderItems.TipeBrg = models.ReadyStock
		}
		qry := `CALL dbSalesOrderAdd(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?, ?, ?)`
		_, err := rp.DB.ExecContext(ctx, qry,
			SalesOrder.UID,
			SalesOrder.KodePT,
			SalesOrder.KodeSales,
			SalesOrder.NoOrder,
			SalesOrder.NoSO,
			SalesOrder.KodeCust,
			SalesOrder.NamaCust,
			SalesOrder.TglSO,
			SalesOrder.TglJT,
			SalesOrder.Nilai,
			SalesOrder.Keterangan,
			SalesOrder.ApprovalStatus,
			OrderItems.TipeBrg,
			OrderItems.NoUrut,
			OrderItems.KodeBrg,
			OrderItems.NamaBrg,
			OrderItems.Satuan,
			OrderItems.Jml,
			OrderItems.Konv,
			OrderItems.Harga,
			OrderItems.DiskonPersen,
			OrderItems.DiskonRupiah,
			OrderItems.TipeDiskon,
			OrderItems.HargaDiskon,
			OrderItems.Total,
		)
		if err != nil {
			log.Println(err)
			return nil, 0, err
		}
	}

	NewSalesOrder, recCount, err := rp.SalesOrderGetByUID(SalesOrder.UID)
	if err != nil {
		return nil, 0, err
	} else if recCount == 0 {
		return nil, 0, errors.New("No record found")
	}

	return NewSalesOrder, 1, nil
}
func (rp *portalDBRepo) SalesOrderUpdate(sKodePT string, sKodeSales string, SalesOrder models.OrderData) (*models.OrderData, int, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()

	_, recCount, err := rp.SalesOrderGetByUID(SalesOrder.UID)
	if err != nil {
		return nil, 0, err
	}
	if recCount == 0 {
		return nil, 0, errors.New("no record to update")
	}

	qry := `DELETE FROM tr_so_d WHERE UID=?`
	_, err = rp.DB.ExecContext(ctx, qry, SalesOrder.UID)
	if err != nil {
		return nil, 0, err
	}
	SalesOrder.KodeSales = sKodeSales
	SalesOrder.KodePT = sKodePT
	for _, OrderItems := range SalesOrder.Items {
		//TODO what should we call here, dbSalesOrderAdd or Update?
		if OrderItems.KodeBrg == "" {
			OrderItems.TipeBrg = models.NewRequest
		} else {
			OrderItems.TipeBrg = models.ReadyStock
		}
		qry := `CALL dbSalesOrderAdd(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?, ?, ?)`
		_, err := rp.DB.ExecContext(ctx, qry,
			SalesOrder.UID,
			SalesOrder.KodePT,
			SalesOrder.KodeSales,
			SalesOrder.NoOrder,
			SalesOrder.NoSO,
			SalesOrder.KodeCust,
			SalesOrder.NamaCust,
			SalesOrder.TglSO,
			SalesOrder.TglJT,
			SalesOrder.Nilai,
			SalesOrder.Keterangan,
			SalesOrder.ApprovalStatus,
			OrderItems.TipeBrg,
			OrderItems.NoUrut,
			OrderItems.KodeBrg,
			OrderItems.NamaBrg,
			OrderItems.Satuan,
			OrderItems.Jml,
			OrderItems.Konv,
			OrderItems.Harga,
			OrderItems.DiskonPersen,
			OrderItems.DiskonRupiah,
			OrderItems.TipeDiskon,
			OrderItems.HargaDiskon,
			OrderItems.Total,
		)
		if err != nil {
			log.Println(err)
			return nil, 0, err
		}
	}

	NewSalesOrder, recCount, err := rp.SalesOrderGetByUID(SalesOrder.UID)
	if err != nil {
		return nil, 0, err
	} else if recCount == 0 {
		return nil, 0, errors.New("No record found")
	}

	return NewSalesOrder, 1, nil
}

func (rp *portalDBRepo) SalesOrderDelete(sKodePT string, sUID string) error {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()

	qry := `DELETE FROM tr_so_d WHERE KodePT=? AND UID=?`
	_, err := rp.DB.ExecContext(ctx, qry, sKodePT, sUID)
	if err != nil {
		log.Println(err)
		return err
	}
	qry = `DELETE FROM tr_so_m WHERE KodePT=? AND UID=?`
	_, err = rp.DB.ExecContext(ctx, qry, sKodePT, sUID)
	if err != nil {
		log.Println(err)
		return err
	}
	return nil
}

func (rp *portalDBRepo) SalesPaymentDelete(sKodePT string, sUID string) error {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()

	qry := `DELETE FROM tr_bayar_d WHERE  UID=?`
	_, err := rp.DB.ExecContext(ctx, qry, sUID)
	if err != nil {
		log.Println(err)
		return err
	}
	qry = `DELETE FROM tr_bayar_m WHERE KodePT=? AND UID=?`
	_, err = rp.DB.ExecContext(ctx, qry, sKodePT, sUID)
	if err != nil {
		log.Println(err)
		return err
	}
	return nil
}

func (rp *portalDBRepo) PaymentListGetBySales(sKodePT string, sKodeSales string) (*[]models.PaymentData, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()
	qry := `CALL dbPaymentListGetBySales(?,? )`

	row, err := rp.DB.QueryContext(ctx, qry, sKodePT, sKodeSales)
	if err != nil {
		log.Println(err)
		return nil, err
	}

	var PaymentData []models.PaymentData
	var PaymentDetail []models.PaymentDetail

	defer func() {
		err := row.Close()
		if err != nil {
			log.Println("Error closing rows!")
		}
	}()
	var PrevUID string

	for row.Next() {
		var Payment models.PaymentData
		var PaymentTerm models.PaymentDetail
		err = row.Scan(
			&Payment.UID,
			&Payment.KodePT,
			&Payment.KodeSales,
			&Payment.NoTransaksi,
			&Payment.TglBayar,
			&Payment.NoBayar,
			&Payment.NoJual,
			&Payment.Nilai,
			&Payment.KodeCust,
			&Payment.Keterangan,
			&Payment.ApprovalStatus,
			&PaymentTerm.NoUrut,
			&PaymentTerm.KodeBayar,
			&PaymentTerm.NoBG,
			&PaymentTerm.Nilai,
		)
		if err != nil {
			log.Println(err)
			return nil, err
		}
		if PrevUID != Payment.UID {
			PaymentDetail = nil
		}
		PaymentDetail = append(PaymentDetail, PaymentTerm)

		if PrevUID != Payment.UID {
			Payment.Items = PaymentDetail
			PaymentData = append(PaymentData, Payment)
		} else {
			Payment.Items = PaymentDetail
			PaymentData[len(PaymentData)-1] = Payment
		}
		PrevUID = Payment.UID
	}
	return &PaymentData, nil
}
func (rp *portalDBRepo) PaymentGetBySalesUID(sUID string, sKodeSales string) (*models.PaymentData, int, error) {
	ctx, cancel := context.WithTimeout(context.Background(), 5*rp.App.ContextTime)
	defer cancel()
	log.Println(sUID, sKodeSales)
	qry := `CALL dbPaymentGetBySalesUID(?,?)`
	row, err := rp.DB.QueryContext(ctx, qry, sUID, sKodeSales)

	var PaymentData []models.PaymentDetail

	defer func() {
		err := row.Close()
		if err != nil {
			log.Println("Error closing rows!")
		}
	}()
	var Payment models.PaymentData
	for row.Next() {
		var PaymentTerm models.PaymentDetail
		err = row.Scan(
			&Payment.UID,
			&Payment.KodePT,
			&Payment.KodeSales,
			&Payment.NoTransaksi,
			&Payment.TglBayar,
			&Payment.NoBayar,
			&Payment.NoJual,
			&Payment.Nilai,
			&Payment.KodeCust,
			&Payment.NamaCust,
			&Payment.Keterangan,
			&Payment.ApprovalStatus,
			&PaymentTerm.NoUrut,
			&PaymentTerm.KodeBayar,
			&PaymentTerm.NoBG,
			&PaymentTerm.Nilai)
		if err != nil {
			log.Println(err)
			return nil, 0, err
		}

		PaymentData = append(PaymentData, PaymentTerm)
		Payment.Items = PaymentData
	}
	return &Payment, 1, nil
}
func (rp *portalDBRepo) PaymentGetByUID(sUID string) (*models.PaymentData, int, error) {
	ctx, cancel := context.WithTimeout(context.Background(), 5*rp.App.ContextTime)
	defer cancel()

	qry := `CALL dbPaymentGetByUID(?,?)`
	row, err := rp.DB.QueryContext(ctx, qry, sUID)

	var PaymentData []models.PaymentDetail

	defer func() {
		err := row.Close()
		if err != nil {
			log.Println("Error closing rows!")
		}
	}()
	var Payment models.PaymentData
	for row.Next() {
		var PaymentTerm models.PaymentDetail
		err = row.Scan(
			&Payment.UID,
			&Payment.KodePT,
			&Payment.KodeSales,
			&Payment.NoTransaksi,
			&Payment.TglBayar,
			&Payment.NoBayar,
			&Payment.NoJual,
			&Payment.Nilai,
			&Payment.KodeCust,
			&Payment.Keterangan,
			&Payment.ApprovalStatus,
			&PaymentTerm.NoUrut,
			&PaymentTerm.KodeBayar,
			&PaymentTerm.NoBG,
			&PaymentTerm.Nilai)
		if err != nil {
			log.Println(err)
			return nil, 0, err
		}

		PaymentData = append(PaymentData, PaymentTerm)
		Payment.Items = PaymentData
	}
	return &Payment, 1, nil
}
func (rp *portalDBRepo) PaymentGetBySalesNoTransaksi(sKodePT string, sNoTransaksi string, sKodeSales string) (*models.PaymentData, int, error) {
	ctx, cancel := context.WithTimeout(context.Background(), 5*rp.App.ContextTime)
	defer cancel()

	qry := `CALL dbGetSalesPaymentRecordByNoTransaksi(?,?,?)`
	row, err := rp.DB.QueryContext(ctx, qry, sKodePT, sNoTransaksi, sKodeSales)
	if err != nil {
		return nil, 0, err
	}
	var PaymentDetails []models.PaymentDetail

	defer func() {
		err := row.Close()
		if err != nil {
			log.Println("Error closing rows!")
		}
	}()
	var Payment models.PaymentData
	for row.Next() {
		var PaymentTerm models.PaymentDetail
		err = row.Scan(
			&Payment.UID,
			&Payment.KodePT,
			&Payment.KodeSales,
			&Payment.NoTransaksi,
			&Payment.TglBayar,
			&Payment.NoBayar,
			&Payment.NoJual,
			&Payment.Nilai,
			&Payment.KodeCust,
			&Payment.Keterangan,
			&Payment.ApprovalStatus,
			&PaymentTerm.NoUrut,
			&PaymentTerm.KodeBayar,
			&PaymentTerm.NoBG,
			&PaymentTerm.Nilai)
		if err != nil {
			log.Println(err)
			return nil, 0, err
		}

		PaymentDetails = append(PaymentDetails, PaymentTerm)
		Payment.Items = PaymentDetails
	}
	return &Payment, 1, nil
}
func (rp *portalDBRepo) PaymentAdd(sKodePT string, sKodeSales string, PaymentData models.PaymentData) (*models.PaymentData, int, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()

	if PaymentData.UID == "" {
		sUID, sNomor, err := rp.NoBaruWithUID("PY", time.Now())
		if err != nil {
			return nil, 0, err
		}
		PaymentData.UID = sUID
		PaymentData.KodePT = sKodePT
		PaymentData.KodeSales = sKodeSales
		PaymentData.NoTransaksi = sNomor
	}
	qry := `DELETE FROM tr_bayar_d WHERE UID=?`
	_, err := rp.DB.ExecContext(ctx, qry, PaymentData.UID)
	if err != nil {
		return nil, 0, err
	}
	for _, PaymentTerm := range PaymentData.Items {
		log.Println(PaymentData)
		qry := `CALL dbPaymentAdd(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)`
		_, err := rp.DB.ExecContext(ctx, qry,
			PaymentData.UID,
			PaymentData.KodePT,
			PaymentData.KodeSales,
			PaymentData.NoTransaksi,
			PaymentData.TglBayar,
			PaymentData.NoBayar,
			PaymentData.NoJual,
			PaymentData.Nilai,
			PaymentData.KodeCust,
			PaymentData.NamaCust,
			PaymentData.Keterangan,
			PaymentData.ApprovalStatus,
			PaymentTerm.NoUrut,
			PaymentTerm.KodeBayar,
			PaymentTerm.NoBG,
			PaymentTerm.Nilai,
		)
		if err != nil {
			log.Println(err)
			return nil, 0, err
		}
	}

	NewPaymentData, recCount, err := rp.PaymentGetBySalesUID(PaymentData.UID, PaymentData.KodeSales)
	if err != nil {
		return nil, 0, err
	} else if recCount == 0 {
		return nil, 0, errors.New("No record found")
	}

	return NewPaymentData, 1, nil
}
func (rp *portalDBRepo) PaymentUpdate(sKodePT string, sKodeSales string, PaymentData models.PaymentData) (*models.PaymentData, int, error) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()

	OldPaymentRecord, recCount, err := rp.PaymentGetBySalesUID(PaymentData.UID, sKodeSales)
	if err != nil {
		return nil, 0, err
	}
	if recCount == 0 {
		return nil, 0, errors.New("no record to update")
	}
	if OldPaymentRecord.ApprovalStatus == models.Approved {
		return nil, 0, errors.New("payment is already approved and cannot be changed.")
	}

	qry := `DELETE FROM tr_bayar_d WHERE UID=?`
	_, err = rp.DB.ExecContext(ctx, qry, PaymentData.UID)
	if err != nil {
		return nil, 0, err
	}
	PaymentData.KodeSales = sKodeSales
	PaymentData.KodePT = sKodePT
	for _, PaymentTerm := range PaymentData.Items {
		qry := `CALL dbPaymentUpdate(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)`
		_, err := rp.DB.ExecContext(ctx, qry,
			PaymentData.UID,
			PaymentData.KodePT,
			PaymentData.KodeSales,
			PaymentData.NoTransaksi,
			PaymentData.TglBayar,
			PaymentData.NoBayar,
			PaymentData.NoJual,
			PaymentData.Nilai,
			PaymentData.KodeCust,
			PaymentData.Keterangan,
			PaymentData.ApprovalStatus,
			PaymentTerm.NoUrut,
			PaymentTerm.KodeBayar,
			PaymentTerm.NoBG,
			PaymentTerm.Nilai,
		)
		if err != nil {
			log.Println(err)
			return nil, 0, err
		}
	}

	NewPayment, recCount, err := rp.PaymentGetBySalesUID(PaymentData.UID, PaymentData.KodeSales)
	if err != nil {
		return nil, 0, err
	} else if recCount == 0 {
		return nil, 0, errors.New("No record found")
	}
	return NewPayment, 1, nil
}
func (rp *portalDBRepo) PaymentApprove(sUID string, ApprovalStatus models.ApprovalStatus) (error, *models.ApprovalStatus) {
	ctx, cancel := context.WithTimeout(context.Background(), rp.App.ContextValue*rp.App.ContextTime)
	defer cancel()

	qry := `CALL dbApprovePayment(?)`
	var ApprovalResult models.ApprovalStatus
	err := rp.DB.QueryRowContext(ctx, qry, sUID).Scan(&ApprovalResult)
	if err != nil {
		return err, nil
	}
	return nil, &ApprovalResult
}
