package repository

import (
	"github.com/violendy/SalesPortal/internal/models"
	"time"
)

type WAGatewayRepo interface {
	SendMessage(sNoHP string, sMessage string) error
	FormatPhoneToJID(sNoHP string) string
}
type LibsFunction interface {
}
type NKDatabaseRepo interface {
	GetSyncSales(sKodePT string, sTransID string, sSales string) (int, error)

	RKSListGetBySales(skodePT string, sKodeSales string) ([]models.RKSData, error)
	RKSListGet(skodePT string) ([]models.RKSData, error)

	CustomerListGet(sKodePT string) ([]models.CustomerData, error)
	CustomerListGetBySales(sKodePT string, sKodeSales string) ([]models.CustomerData, error)
	CustomerAdd(sKodePT string, data models.CustomerData) error
	SalesGetProduct(sKodePT string) (*[]models.Product, int, error)
	PaymentAdd(sKodePT string, data models.PaymentData) error
}

type PortalDatabaseRepo interface {
	NoBaruWithUID(TransID string, Tgl time.Time) (string, string, error)
	GetFile(md5checksum string) (string, error)
	NewFile(filename string, md5checksum string) (string, error)

	UserLogin(credential *models.LoginCredential) (models.UserLoginResponse, error)
	UserLogout(sToken string) (bool, error)
	UserCheckToken(sToken string) bool
	UserGetByToken(sToken string) (models.UserData, error)
	GetSalesByNoHP(sNoHP string) (*models.SalesData, error)
	SalesLogin(credential *models.LoginCredential) (models.SalesLoginResponse, error)
	SalesLogout(sToken string) (bool, error)
	SalesAdd(sKodePT string, Sales models.SalesData) error
	SalesUpdate(sKodePT string, Sales models.SalesData) error
	SalesListGet(sKodePT string) (*[]models.SalesData, int, error)
	SalesCheckIn(sKodePT string, CheckInData models.SalesCheckIn) (*models.SalesVisit, error)
	SalesCheckOut(sKodePT string, CheckOutData models.SalesCheckOut) (*models.SalesVisit, error)
	SalesGetCustomerInArea(sKodePT string, sKodeSales string, sCurrentLocation models.Location) (error, int, *[]models.CustomerData)
	SalesGetByToken(sToken string) (models.SalesData, error)
	SalesGetVisit(sKodePT string, sKodeSales string) (*[]models.SalesVisit, int, error)
	SalesGetProduct(sKodePT string) (*[]models.Product, int, error)

	SalesCheckToken(sToken string) bool

	NeedSync(sKodePT string, sKodeSales string, sTransID string, CounterToCheck int) (bool, error)
	UpdateSyncID(sKodePT string, sKodeSales string, sTransID string, NewID int) error

	ChangePassword(sKodePT string, newcredential *models.ChangeCredential) (models.ChangePasswordResponse, error)
	RequestResetPassword(sNoHP string) (string, string, error)

	RKSSetData(sKodePT string, RKSData []models.RKSData) (int, error)
	RKSListGetBySales(skodePT string, sKodeSales string) (*[]models.RKSData, error)
	RKSListGet(skodePT string) (*[]models.RKSData, error)

	CustomerSet(sKodePT string, CustomerData []models.CustomerData) (int, error)
	CustomerListGet(sKodePT string) (*[]models.CustomerData, error)
	CustomerGetByKodeCust(sKodePT string, sKodeCust string) (*models.CustomerData, int, error)
	CustomerGetBySalesKodeCust(sKodePT string, sKodeSales string, sKodeCust string) (*models.CustomerData, int, error)
	CustomerGetByUID(sUID string) (*models.CustomerData, int, error)
	CustomerAdd(sKodePT string, Customer models.CustomerData) (*models.CustomerData, error)
	CustomerUpdate(sKodePT string, Customer models.CustomerData) (*models.CustomerData, int, error)
	CustomerDelete(sKodePT string, sUID string) error
	CustomerApprove(sUID string, ApprovalStatus models.ApprovalStatus) (error, *models.ApprovalStatus)
	CustomerListGetBySales(sKodePT string, sKodeSales string) (*[]models.CustomerData, error)

	PaymentApprove(sUID string, ApprovalStatus models.ApprovalStatus) (error, *models.ApprovalStatus)
	PaymentGetByUID(sUID string) (*models.PaymentData, int, error)
	PaymentListGetBySales(sKodePT string, sKodeSales string) (*[]models.PaymentData, error)
	PaymentGetBySalesUID(sUID string, sKodeSales string) (*models.PaymentData, int, error)
	PaymentGetBySalesNoTransaksi(sKodePT string, sNoTransaksi string, sKodeSales string) (*models.PaymentData, int, error)
	PaymentAdd(sKodePT string, sKodeSales string, PaymentData models.PaymentData) (*models.PaymentData, int, error)
	PaymentUpdate(sKodePT string, sKodeSales string, PaymentData models.PaymentData) (*models.PaymentData, int, error)
	SalesPaymentDelete(sKodePT string, sUID string) error

	SalesOrderListGetBySales(sKodePT string, sKodeSales string) (*[]models.OrderData, error)
	SalesOrderGetByUID(sUID string) (*models.OrderData, int, error)
	SalesOrderGetByNoOrder(sKodePT string, sNoOrder string, sKodeSales string) (*models.OrderData, int, error)
	SalesOrderAdd(sKodePT string, sKodeSales string, SalesOrder models.OrderData) (*models.OrderData, int, error)
	SalesOrderUpdate(sKodePT string, sKodeSales string, SalesOrder models.OrderData) (*models.OrderData, int, error)
	SalesOrderDelete(sKodePT string, sUID string) error
}
