package main

import (
	"github.com/go-chi/chi/v5"
	"github.com/go-chi/cors"
	"github.com/violendy/SalesPortal/internal/config"
	"github.com/violendy/SalesPortal/internal/handler"
	"net/http"
)

func routes(app *config.AppConfig) http.Handler {
	//mux := pat.New()
	//
	//mux.Get("/dashboard", http.HandlerFunc(handler.Repo.Dashboard))
	//mux.Get("/test", http.HandlerFunc(handler.Repo.Test))
	//log.Println("InProduction : ", app.InProduction)
	mux := chi.NewRouter()
	mux.Use(cors.Handler(cors.Options{
		// AllowedOrigins:   []string{"https://foo.com"}, // Use this to allow specific origin hosts
		AllowedOrigins: []string{"https://*", "http://*"},
		// AllowOriginFunc:  func(r *http.Request, origin string) bool { return true },
		AllowedMethods:   []string{"GET", "POST", "PUT", "DELETE", "OPTIONS"},
		AllowedHeaders:   []string{"X-PINGOTHER", "Accept", "Authorization", "Content-Type", "X-CSRF-Token"},
		AllowCredentials: false,
		MaxAge:           300000, // Maximum value not ignored by any of major browsers
	}))
	//mux.NotFound(func(writer http.ResponseWriter, request *http.Request) {
	//	http.Redirect(writer, request, "http://localhost:8080", )
	//})
	mux.Use(LogRequest)
	//mux.Use(NoSurf)
	//mux.Use(SessionLoad)
	//mux.Use(middleware.Recoverer)
	//mux.Use(middleware.CleanPath)

	//mux.Post("/api/notification/wa/customer/payment/{KodePT}", handler.Repo.WaPaymentNotif)
	//mux.Post("/api/notification/wa/customer/salesorder/{KodePT}", handler.Repo.WaSalesOrderNotif)

	mux.Post("/api/portal/login", handler.Repo.UserLogin)
	//mux.Post("/api/portal/approve/loginreset/{KodePT}", handler.Repo.ApproveResetLogin)

	mux.Get("/api/portal/customer/{KodePT}", handler.Repo.PortalGetAllCustomerList)
	mux.Get("/api/portal/customer/{KodePT}/{KodeCust}", handler.Repo.PortalGetCustomerRecordByKodeCust)
	mux.Post("/api/portal/customer", handler.Repo.PortalCreateCustomerRecord)
	mux.Put("/api/portal/customer", handler.Repo.PortalUpdateCustomerRecord)
	//mux.Delete("/api/portal/customer/{ResourceUID}", handler.Repo.DeleteCustomerRecord)
	//mux.Get("/api/portal/visit", handler.Repo.PortalGetVisitList)
	mux.Put("/api/portal/customer/approve/{UID}", handler.Repo.PortalApproveCustomer)
	mux.Put("/api/portal/payment/approve/{KodePT}", handler.Repo.PortalApprovePayment)

	mux.Post("/api/sales/logout", handler.Repo.SalesLogout)
	mux.Post("/api/sales/login", handler.Repo.SalesLogin)
	mux.Post("/api/sales/login/change", handler.Repo.SalesChangePassword)
	mux.Get("/api/sales/login/reset/{NoHP}", handler.Repo.SalesRequestResetPassword)
	mux.Get("/api/sales/rks", handler.Repo.SalesGetRKSList)
	mux.Get("/api/sales/checklocation/{salesLat}/{salesLng}", handler.Repo.SalesGetCurrentLocationCustomer)

	mux.Post("/api/sales/checkin", handler.Repo.SalesCheckIn)
	mux.Post("/api/sales/checkout", handler.Repo.SalesCheckOut)
	mux.Get("/api/sales/visit", handler.Repo.SalesGetVisit)
	mux.Get("/api/sales/produk", handler.Repo.SalesGetProduct)

	mux.Get("/api/sales/customer/list", handler.Repo.SalesGetCustomerList)
	mux.Get("/api/sales/customer/{KodeCust}", handler.Repo.SalesGetCustomerRecordByKodeCust)
	mux.Post("/api/sales/customer", handler.Repo.SalesCreateCustomerRecord)
	mux.Put("/api/sales/customer", handler.Repo.SalesUpdateCustomerRecord)
	mux.Get("/api/sales/customer/pictures/{filename}", handler.Repo.SalesGetPicture)

	mux.Get("/api/sales/order/list", handler.Repo.SalesGetSalesOrderList)
	mux.Get("/api/sales/order/{NoOrder}", handler.Repo.SalesGetSalesOrderRecordByNoOrder)
	mux.Delete("/api/sales/order/delete/{UID}", handler.Repo.SalesDeleteOrderByUID)
	mux.Post("/api/sales/order", handler.Repo.SalesCreateSalesOrderRecord)
	mux.Put("/api/sales/order", handler.Repo.SalesUpdateSalesOrderRecord)

	mux.Get("/api/sales/payment/list", handler.Repo.SalesGetPaymentList)
	mux.Get("/api/sales/payment/{NoTransaksi}", handler.Repo.SalesGetPaymentRecordByNoTransaksi)
	mux.Post("/api/sales/payment", handler.Repo.SalesCreatePaymentRecord)
	mux.Put("/api/sales/payment", handler.Repo.SalesUpdatePaymentRecord)
	mux.Delete("/api/sales/payment/delete/{UID}", handler.Repo.SalesDeletePaymentByUID)

	mux.Get("/api/wa/send/{nohp}/{message}", handler.Repo.SendWhatsApp)
	mux.Get("/", handler.Repo.Home)
	mux.Get("/debug", handler.Repo.ToggleProduction)
	mux.Get("/debug/{val}", handler.Repo.SetContextTimeout)

	prodServer := http.FileServer(http.Dir("./dist/"))
	mux.Handle("/*", prodServer)

	return mux
}
